!==================================================================!
      PROGRAM ncplot
!==================================================================!

! PURPOSE:  produce IGOR-friendly NetCDF and ascii versions
!           of GECKO-A NetCDF box model output
! FIELDS FOR AER-ONLY VERSION: 
!        name, formulaCxHyOzNa, MW
!        funst=A,D,G,H,K,N,O,P,T(# of each),
!        aerconc(time=1,n) 
!        chemical formula
!        PLUS separate results lines for:
!        T, j(no2,o3,hono,hcho,no3),RH,
!        mgas, maer, mwall, OtoC, HtoC, NtoC (each for gas, aer, wall) 
!
! FIELDS FOR COMPLETE VERSION: 
!        name, formulaCxHyOzNa, MW
!        funst=A,D,G,H,K,N,O,P,T(# of each),
!        radical_flag, RO2-type, Gas/Particle_identifier,
!        conc(time=1,n),
!        chemical formula
!        PLUS separate results lines for:
!        T, j(no2,o3,hono,hcho,no3),RH,
!        mgas, maer, mwall, OtoC, HtoC, NtoC (each for gas, aer, wall) 
!
! INPUT: NetCDF GECKO-A boxmodel output outdat.nc
! OUTPUT1: NetCDF IGOR-readable file post.nc
! OUTPUT2: companion ascii file post.ascii

! akparameter is not necessary for NetCDF read: 
! all params are in the outdat.nc file
!
! VERSION UPDATES:
! V0.2, Jan 24, 2019, JMLT, CIRES/NCAR
!      - using module inputs for timing and logging comments
! AUTHOR: Julia Lee-Taylor, NCAR & CU/CIRES
!
!---------------------------------------------------------------------!
      USE ncplot_input_module
      IMPLICIT NONE
!      INTEGER :: i,j,k,l,m,n,p,q
      INTEGER :: j,k
!======================================================================
! USER SUPPLIES OUTPUT TIME DEFINITIONS : REQUIRED PARAMS ARE....
!      INTEGER,PARAMETER :: nrt = (value or equation)
!      REAL,DIMENSION(nrt) :: reftime = (values or expression)
!      CHARACTER(LEN=50) :: creftime = (string)
!======================================================================
!--model output times (informational only)
!      INTEGER,PARAMETER :: opstop = 86400
!      REAL,PARAMETER    :: opstep = 300.     

!--EITHER calculate postprocessed output time array (evenly spaced)
!      INTEGER,PARAMETER :: rtstop = opstop
!      REAL,PARAMETER    :: rtstep = 600.
!      CHARACTER(LEN=50) :: creftime="0-24h x 10m"
!      INTEGER,PARAMETER :: nrt = INT(FLOAT(rtstop)/rtstep)+1
!      REAL,DIMENSION(nrt) :: reftime = rtstep*(/(i,i=0,nrt-1,1)/)

!--OR specify variable spacing output time array (nrt must agree)
! ZP 30 min OHR run: 0-1s @0.01s; 1-2s @0.1s; 2s-3mins @1s; 3-10min @2s;
!!             10-30min @10s. Model OP TS = 0.01s 
!      CHARACTER(LEN=50) :: creftime=
!     &"0-1s,1-2s,2s-1m,1-10m,10-30m x 0.01,0.1,1,2s,10s" 
!      INTEGER,PARAMETER :: nrt = 469
!      REAL,DIMENSION(nrt) :: reftime = rtstep * (/
!     &   (i,i=    0,    99,  10), 
!     &   (j,j=  100,   199,  10),
!     &   (k,k=  200,  5999, 100),
!     &   (l,l= 6000, 59999, 200),
!     &   (m,m=60000,180000,1000) 
!     &                        /)

! END OUTPUT TIME DEFINITIONS
!======================================================================
* HEADER OUTPUT LINES 
      CHARACTER(LEN=16):: version= 'ncplot.f.0.2'
      CHARACTER(LEN=30):: author = 'Julia Lee-Taylor, CIRES & NCAR'
! (read from ../INPUTS/ncplot_inputs_module.f90)
!      CHARACTER(LEN=30):: notes1 = '      '    
!======================================================================
* UNITS FOR OUTPUT QUANTITIES
      INTEGER,PARAMETER :: iunits = 5
      CHARACTER(10) :: cunits
!CASE (1) ! molec/cc   factor = 1.
!CASE (2) ! molec.C/cc factor = REAL(ncarbon(i))     
!CASE (3) ! ppbv       factor = 1./numdens
!CASE (4) ! ppbC       factor = REAL(ncarbon(i))/numdens
!CASE (5) ! ug/m3      factor = wmol(i)*1.66e-12     
!CASE (6) ! ugC/m3     factor = REAL(ncarbon(i))*12.01*1.66e-12 
!======================================================================
      LOGICAL,PARAMETER :: asciiopflag = .TRUE. !(write ascii output)

!--normal definitions

      INTEGER :: i1,i2,it,kt,ndic,ndat,memo
      INTEGER :: AllocateStatus
**DIMENSIONS: TO BE READ FROM NETCDF INPUTS
      INTEGER :: maxsp , mxsat , mxdep , mxro2cl , mdat, mjchrom
      INTEGER :: mxsim 

! other dimensions
      INTEGER,PARAMETER :: maxfun = 15
      INTEGER,PARAMETER :: nro2typ = 9

! non-NetCDF input/output units
      INTEGER,PARAMETER :: uin = 21
      INTEGER,PARAMETER :: uou = 31

* earliest data to output from the result file
      INTEGER,PARAMETER :: mindat= 1!  97  !1

**CHARACTER LENGTHS=========================
* length of string & functionalities list in the dictionary
* name, formula, functionalities list lengths of a given species
      INTEGER,PARAMETER :: ldi=146 ! (long)
      INTEGER,PARAMETER :: lco=6
      INTEGER,PARAMETER :: lfo=120
      INTEGER,PARAMETER :: lfl=15
      INTEGER,PARAMETER :: lmf = 11
      INTEGER,PARAMETER :: llin = 100
* max length of species names
      INTEGER,PARAMETER :: maxlsp=8
      CHARACTER(LEN=1)  :: c1  ! string = ","
      CHARACTER(LEN=3)  :: cipos  ! string = actual length of formula
      CHARACTER(LEN=3)  :: cspos  ! string = actual length of SMILE
      CHARACTER(LEN=2)  :: ctref ! string = length of time descriptor
      CHARACTER(LEN=:),ALLOCATABLE  :: cnout ! string = # of output times

!      CHARACTER(len=ldi) :: dict
      CHARACTER(len=lfo) :: tchem,smile
      CHARACTER(len=maxlsp) :: namex(3)
!      CHARACTER(len=1) :: cf
!      CHARACTER(len=8) :: hdr
      CHARACTER(len=lmf) :: molform ! molecular formula
      CHARACTER(len=llin) :: line ! input from keyfile, README file

! ALLOCATABLE
      CHARACTER(len=lco),ALLOCATABLE,DIMENSION(:) :: name,oname
      CHARACTER(len=lfo),ALLOCATABLE,DIMENSION(:) :: chem,ochem
      CHARACTER(len=lfl),ALLOCATABLE,DIMENSION(:) :: code,ocode
      CHARACTER(len=maxlsp),ALLOCATABLE,DIMENSION(:) :: chrsp,chrsp_H
      CHARACTER(len=maxlsp),ALLOCATABLE,DIMENSION(:,:) :: chrsp_ro2
      INTEGER,ALLOCATABLE,DIMENSION(:,:) :: idchemro2
      INTEGER,ALLOCATABLE,DIMENSION(:) :: satid

! Keeping all mdat values is too memory-intensive. keep only nrt values
! index(nrt) for auxiliary output data (mdat): 
      INTEGER,DIMENSION(nrt) :: idat
      REAL :: time_read
      REAL,DIMENSION(1) :: time1
      REAL,DIMENSION(1,1) :: temp2,rhum2

!=auxiliary output data (nrt)=====================
      REAL,DIMENSION(nrt) :: time,temp,rhum   !,jeff
      REAL,DIMENSION(nrt) :: OHexp,O3exp,NO3exp
      REAL,DIMENSION(nrt) :: jNO2,jNO3,jO3,jHONO,jCH2O,jH2O2
      REAL,DIMENSION(nrt) :: OtoC_gas, OtoC_aer, OtoC_wal
      REAL,DIMENSION(nrt) :: HtoC_gas, HtoC_aer, HtoC_wal
      REAL,DIMENSION(nrt) :: NtoC_gas, NtoC_aer, NtoC_wal
      REAL,DIMENSION(nrt) :: maer_out,mgas_out,mwal_out
      REAL,DIMENSION(nrt) :: ctotgas,corgaer,corgwal
! info from soa output file
      REAL :: soatime, Cstar
      REAL,DIMENSION(nrt) :: ctotaer, Rp, Mp, mwaer

! species output data ! ALLOCATABLE
! note that "Cstar" is really Cstar298
      REAL,ALLOCATABLE,DIMENSION(:) :: wmol,DVa,Cw   
      REAL,ALLOCATABLE,DIMENSION(:) :: Cstar_298,Cstar_T
      REAL,ALLOCATABLE,DIMENSION(:) :: Tb,dB,oTb,odB
      REAL,ALLOCATABLE,DIMENSION(:) :: kHenry,okHenry
      REAL,ALLOCATABLE,DIMENSION(:) :: kwg,kga,okphase
      REAL,ALLOCATABLE,DIMENSION(:) :: dh2odx,odh2odx
      REAL,ALLOCATABLE,DIMENSION(:) :: reacf,oreacf
      INTEGER,ALLOCATABLE,DIMENSION(:) :: ncarbon,nnitrogen
     &                          , noxygen,nhydrogen,nsulfur
      REAL,ALLOCATABLE,DIMENSION(:,:,:) :: cgas3
      REAL,ALLOCATABLE,DIMENSION(:,:) :: bk
      REAL,ALLOCATABLE,DIMENSION(:,:) :: simpgroup,osimpgroup
      REAL,ALLOCATABLE,DIMENSION(:,:) :: cgas,cout
      REAL,ALLOCATABLE,DIMENSION(:,:) :: caer,psat
      REAL,ALLOCATABLE,DIMENSION(:,:) :: cro2

! pvap scheme used to calculate Cstar and wall rate kwg
      CHARACTER(3),PARAMETER :: pvap_scheme = "SIM" ! NAN/SIM

! species output data ! set dimensions
      INTEGER,DIMENSION(maxfun) :: fungrp
      INTEGER :: numsp,lensp  
      INTEGER :: nsat,nhenry
      INTEGER,DIMENSION(nro2typ) :: n_ro2
      INTEGER :: spos,ipos,iend
      INTEGER :: nout
      INTEGER :: jpvap,jhenry
      INTEGER :: idO3,idOH,idNO3

      REAL :: KGWA,woucfa,woucfb
      REAL :: dt,Trb,bk_T,pvap,log10psat
      REAL :: mw_wall = 250.

      CHARACTER(1) w,v
      CHARACTER(2) x,y,z

      INTEGER,DIMENSION(8) :: date_time
      CHARACTER(LEN=10),DIMENSION(3) :: b 
      CHARACTER(LEN=3),DIMENSION(12),PARAMETER :: month=(/'Jan','Feb',
     &  'Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'/)

* selectors for which phase to output
      LOGICAL ginorgonly !(limits gas phase o/p to inoganics only)
      LOGICAL fgcodes !(outputs func groups)
      LOGICAL radicals !(outputs radical info)
      LOGICAL alltimes !(outputs every time data point)
      LOGICAL lasttime  !(outputs only last time data point

      LOGICAL soaf2 !(type of mechanism / boxmod run)
      LOGICAL modern ! "false" "true" has 

      INTEGER,PARAMETER :: nfgrp=15 ! # of functional group types
      INTEGER :: ifg ! 1=inorg, 0=organic
      INTEGER :: radtyp ! 
      INTEGER,ALLOCATABLE,DIMENSION(:) :: perotyp ! 
      INTEGER,ALLOCATABLE,DIMENSION(:) :: iphz ! 1=gas, 2=aer, 3=wall
      INTEGER :: ihdr  !,ityp
      CHARACTER(LEN=3) :: extn

! jvals output info
      INTEGER :: ijO3,ijNO2,ijNO3,ijHONO,ijCH2O,ijH2O2
      INTEGER :: njchrom
      INTEGER,PARAMETER :: mjval=10 ! maximum number of named chromophores 
      REAL,ALLOCATABLE,DIMENSION(:,:) :: jchrom,ojchrom
! labels for numbered chromophores
      INTEGER,PARAMETER :: ljlab=5
      INTEGER,ALLOCATABLE,DIMENSION(:) :: idchrom
      CHARACTER(LEN=ljlab),ALLOCATABLE,DIMENSION(:) :: jlabel 

! header lines in Henry file
      INTEGER,PARAMETER :: nhdr_henry = 10

! cutoff filter for output
      !REAL,PARAMETER:: cutoff=(1.e-20)
      REAL,PARAMETER :: cutoff = 0.0

! species to ignore in mass, ratios
      CHARACTER(LEN=6),PARAMETER :: spignore="N01003"

      REAL,PARAMETER :: Natm= 2.46e19! Loschmidt's #, molec cm-3 (1 atm, 298 K)
      REAL,PARAMETER :: Ratm= 0.0820578 ! atm L K-1 mol-1
      REAL,PARAMETER :: Navo= 6.02214e23 ! Avogadro constant, molec mol-1

! isp range for species loops (zero values here give 1,numsp)
      INTEGER,PARAMETER :: ilo = 0, ihi = 0

! NetCDF params/variables
      INTEGER :: ncid_in,ncid_out
      CHARACTER(LEN=15) :: attname
      CHARACTER(LEN=70) :: att_txt,cit_1,cit_2,cit_3

! variables & flags from indat.key
      INTEGER :: soa_fg, pvap_fg, dyn_fg, wall_fg, OFR_fg, seedtyp_fg
      INTEGER :: isomer_fg, dimer_fg, autoox_fg, high_NOxfg, zero_NOxfg
      INTEGER :: ICHAM, flgdhf, ro2_fg, depos_fg
      REAL :: f185, f254, WATR, SEED, NVMW, NVRO, NDEN, GAMM, TLEN
      
! chamber selector: 
! 1=Mastunaga/Ziemann(2010); 2=Krechmer(2016); 3=Jimenez_CU
!      INTEGER,PARAMETER :: ICHAM = 3

* -----------------------------
* INITIALIZE AND READ ALL DATA
* -----------------------------
      !STOP
! --- initialise logical flags ---
      modern = .FALSE.
      soaf2 = .FALSE.   ! FALSE: read *.ppf and *.ppa files
      ginorgonly = .FALSE. ! TRUE = aer for inorgs only
      fgcodes = .FALSE.
      radicals = .FALSE.
      alltimes = .FALSE.
      lasttime = .FALSE.

! --- EDIT logical flags ---
!      modern = .TRUE.  ! TRUE: ppa file has an extra value in header
      !soaf2 = .TRUE.   ! TRUE: read *.ppf file only
      !ginorgonly = .TRUE. ! TRUE = gas for inorgs only
      !alltimes = .TRUE.
      !lasttime = .TRUE.
      radicals = .TRUE.
      fgcodes = .TRUE.

! --- initialize units ---
      SELECT CASE (iunits)
        CASE (1) 
          cunits ='molec/cc'
        CASE (2)  
          cunits ='molec.C/cc'
        CASE (3) 
          cunits ='ppbv'
        CASE (4) 
          cunits ='ppbC'
        CASE (5) 
          cunits ='ug/m3'
        CASE (6) 
          cunits ='ugC/m3'
        END SELECT

* -----------------------------
! --- initialize variables ---
      namex=' '
      ihdr = 0
      fungrp=0
      radtyp = 0
      n_ro2 = 0.
      corgwal = 0.
      corgaer = 0.
      ctotgas = 0.
      ctotaer = 0.
      soatime = 0.
      Rp = 0.
      mwaer = 0.
      mgas_out = 0.
      maer_out = 0.
      mwal_out = 0.
      OtoC_gas = 0. 
      OtoC_aer = 0.
      OtoC_wal = 0.
      HtoC_gas = 0. 
      HtoC_aer = 0.
      HtoC_wal = 0.
      NtoC_gas = 0. 
      NtoC_aer = 0.
      NtoC_wal = 0.
      jNO2 = 0.
      jNO3 = 0.
      jO3 = 0.
      jHONO = 0.
      jCH2O = 0.
      jH2O2 = 0.
      OHexp = 0.
      O3exp = 0.
      NO3exp = 0.
! --- initialize input file variables ---
      autoox_fg = 0
      depos_fg = 0
      dimer_fg = 0
      dyn_fg = 1
      flgdhf = 0
      high_NOxfg = 0
      zero_NOxfg = 0
      ICHAM = 0
      isomer_fg = 0
      OFR_fg = 0
      pvap_fg = 2
      ro2_fg = 1
      seedtyp_fg = 0
      soa_fg = 0
      wall_fg = 0

      f185 = 0.
      f254 = 0.
      GAMM = 1.
      KGWA = 0.
      !NDEN = 2.46e+19
      NDEN = 2.04e+19
      SEED = 10.
      NVMW = 426. !DOS
      NVRO = 1.25e-5 !(cm) = 125nm
! (can't initialize allocatable variables)
      !name=' '
      !chem=' '
      !code=' '
      !oname=' '
      !ochem=' '
      !ocode=' '
      !chrsp=' '
      !chrsp_H=' '
      !wmol = 0.
      !DVa = 0.
      !ncarbon = 0
      !noxygen = 0
      !nnitrogen = 0
      !nhydrogen = 0
      !iphz = 0
      !perotyp = 0
      !cro2 = 0.
      !oTb = 0.
      !Tb = 0.
      !dB = 0
      !dh2odx = 0
      !kHenry = 0
      !kwg = 0
      !reacf = 0
      !Cstar_298 = 0
      !cout = 0.
* -------------------------------------------
! --- WRITE PARAMS TO SCREEN FOR CHECKING ---
* -------------------------------------------

      PRINT*,"Press ENTER if satisfied with compilation"
      PRINT*,"   or Ctrl-C to abandon"
!      READ(5,*)
!--------------------------------------------------
      PRINT*,"--------------------"
      PRINT*," CHECK SWITCHES NOW"
      PRINT*,"--------------------"
      PRINT*," modern = ",modern
      PRINT*," soaf2 = ",soaf2
      PRINT*," ginorgonly = ",ginorgonly
      PRINT*," fgcodes = ",fgcodes
      PRINT*," radicals = ",radicals
      PRINT*," alltimes = ",alltimes
      PRINT*," lasttime = ",lasttime
      PRINT*," ilo (first o/p sp) = ",ilo
      PRINT*," ihi (last o/p sp) = ",ihi
      PRINT*," ICHAM (chamber index) = ",ICHAM
      PRINT*,"--------------------"
      PRINT*,"Press ENTER when satisfied"
      PRINT*,"   or Ctrl-C to abandon"
!      READ(5,*)
!--------------------------------------------------
      PRINT*," nrt = ",nrt
      PRINT*," reftime = ",reftime
      PRINT*,"--------------------"
      PRINT*,"Press ENTER when satisfied"
      PRINT*,"   or Ctrl-C to abandon"
!      READ(5,*)
!--------------------------------------------------

* ----------------------------------
! OPEN NetCDF files for readin and output 
* ----------------------------------
     
      CALL open_ncfile_readonly("outdat.nc",ncid_in)
      PRINT*," opened nc input file"

      CALL open_ncfile_new("post.nc",ncid_out)
      PRINT*," opened nc output file"

      IF(asciiopflag) THEN
      OPEN(uou,FILE='post.ascii')
      WRITE(uou,*)'---------------GECKO-A MODEL OUTPUT-----------------'
      ENDIF

* ---------------------------------------------
! --- NetCDF: find input dimensionsi, allocate arrays ---
* ---------------------------------------------
      CALL eznc_get_dimension(ncid_in,"maxsp",maxsp)
      CALL eznc_get_dimension(ncid_in,"ntout",mdat)
      CALL eznc_get_dimension(ncid_in,"mxdep",mxdep)
      CALL eznc_get_dimension(ncid_in,"mxsim",mxsim)
      CALL eznc_get_dimension(ncid_in,"mxsat",mxsat)
      CALL eznc_get_dimension(ncid_in,"mxro2cl",mxro2cl)
      CALL eznc_get_dimension(ncid_in,"nchrom",mjchrom)
             njchrom = mjchrom

! maxsp
      ALLOCATE ( wmol(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( Cstar_298(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( Cstar_T(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( DVa(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( Cw(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( oTb(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( oDb(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( osimpgroup(maxsp,31), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( kHenry(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( okHenry(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( kwg(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( kga(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( okphase(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( dh2odx(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( odh2odx(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( reacf(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( oreacf(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( ncarbon(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( nnitrogen(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( noxygen(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( nhydrogen(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( nsulfur(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( perotyp(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( iphz(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"

      ALLOCATE ( name(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( oname(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( chem(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( ochem(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( code(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( ocode(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( chrsp(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( chrsp_H(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( satid(maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( chrsp_ro2(maxsp,9), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( idchemro2(mxro2cl,9), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"

      ALLOCATE ( cgas3(1,1,maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( cgas(nrt,maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( cout(nrt,maxsp), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"

! mxsat
      ALLOCATE ( Tb(mxsat), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( dB(mxsat), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( bk(mxsim,4), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( simpgroup(mxsat,mxsim), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( caer(nrt,mxsat), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( psat(nrt,mxsat), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"

      ALLOCATE ( cro2(nrt,nro2typ+1), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
! mjchrom
      ALLOCATE ( idchrom(mjchrom), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory ***"
      ALLOCATE ( jlabel(mjchrom), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory, jlabel ***"
      ALLOCATE ( jchrom(mjchrom,1), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory, jchrom ***"
      ALLOCATE ( ojchrom(nrt,mjchrom), STAT = AllocateStatus )
      IF (AllocateStatus /= 0) STOP "*** Not enough memory, ojchrom ***"

* ---------------------------------------------
! --- NetCDF: define output dimensions ---
* ---------------------------------------------

      CALL eznc_def_dim(ncid_out,"maxsp",maxsp)
      CALL eznc_def_dim(ncid_out,"nrt",nrt)
      CALL eznc_def_dim(ncid_out,"lco",lco)      
      CALL eznc_def_dim(ncid_out,"lmf",lmf)      
      CALL eznc_def_dim(ncid_out,"lfo",lfo)      
      CALL eznc_def_dim(ncid_out,"njchrom",mjchrom) 
      CALL eznc_def_dim(ncid_out,"nfgrp",nfgrp) 

* ---------------------------------------------
! --- read and reproduce run documentation ---
* ---------------------------------------------
      OPEN(uin,file='outdat.README')
        DO WHILE(.TRUE.)
          READ(uin,'(a100)',end=998) line
          i=INDEX(line,'=')

          IF(i.NE.1.AND.i.NE.0.AND.INDEX(line,'(').NE.1)THEN

            PRINT*,line
            IF(asciiopflag) WRITE(uou,*)line(1:LEN_TRIM(line))

            attname(:) = " "
            attname=line(1:LEN_TRIM(line(1:i-1)))
            j = INDEX(attname(1:LEN_TRIM(line(1:i-1))),' ')
            IF(j.GT.0) attname(j:j)="_"
            PRINT*,"attname ",attname(1:LEN_TRIM(attname))

            att_txt=" "
            att_txt(1:50)=line(i+1:i+50)
            PRINT*,"att_txt ",att_txt(1:LEN_TRIM(att_txt))

            CALL eznc_def_globalatt(ncid_out,attname,att_txt)

          ENDIF
        ENDDO
998   CLOSE(uin)

* ----------------------------------
! --- WRITE TOP OF ASCII HEADER ---
! --- & NetCDF GLOBAL ATTRIBUTES ---
* ----------------------------------

      CALL date_and_time(b(1),b(2),b(3),date_time)

      WRITE(att_txt,'(a3,1x,i2,a1,1x,i4,a2,2(i2,a1),i2,a4,a3)')
     &             month(date_time(2)),date_time(3),',',date_time(1),
     &             ', ',date_time(5),':',date_time(6),':',date_time(7),
     &             " UTC",b(3)(1:3)

      cit_1 = 'Aumont et al https://doi.org/10.5194/acp-5-2497-2005'
      cit_2 = 'Camredon etc https://doi.org/10.5194/acp-7-5599-2007'
      cit_3 = 'Valorso etc https://doi.org/10.5194/acp-11-6895-2011'

!---- NetCDF o/p ----
      CALL eznc_def_globalatt(ncid_out,"ncfile_extracted",att_txt(1:29))
      CALL eznc_def_globalatt(ncid_out,"ncfile_version",version)
      CALL eznc_def_globalatt(ncid_out,"ncfile_timing",
     &                               creftime(1:len_trim(creftime)))
      CALL eznc_def_globalatt(ncid_out,"other_notes",notes1)
      CALL eznc_def_globalatt(ncid_out,"author_affil",author)
      CALL eznc_def_globalatt(ncid_out,"MAIN_CITATION",cit_1)
      CALL eznc_def_globalatt(ncid_out,"UPDATES_1",cit_2)
      CALL eznc_def_globalatt(ncid_out,"UPDATES_2",cit_3)

* ---- ascii o/p ----

      IF(asciiopflag)THEN
      WRITE(uou,'(a19,a3,1x,i2,a1,1x,i4,a2,2(i2,a1),i2,a4,a3)')
     &           ' ascii extracted : ',
     &             month(date_time(2)),date_time(3),',',date_time(1),
     &             ', ',date_time(5),':',date_time(6),':',date_time(7),
     &             " UTC",b(3)(1:3)

      WRITE(uou,*)'ascii version : '// version(1:LEN_TRIM(version))
      WRITE(ctref,'(i2)') len_trim(creftime)
      WRITE(uou,'(a17,a'//ctref//')')
     &           ' ascii timing  : ',creftime(1:len_trim(creftime))
      WRITE(uou,*)'other notes   : '// notes1(1:len_trim(notes1))
      WRITE(uou,*)'author,affil  : '// author(1:len_trim(author))

      WRITE(uou,*)'-------------------ATTRIBUTION----------------------'

      WRITE(uou,*)' MAIN CITATION:'
      WRITE(uou,*) cit_1(1:LEN_TRIM(cit_1))
      WRITE(uou,*)' UPDATES:'
      WRITE(uou,*) cit_2(1:LEN_TRIM(cit_2))
      WRITE(uou,*) cit_3(1:LEN_TRIM(cit_3))

      WRITE(uou,*)'-------SIMULATION INPUTS = FILE "indat_*.key"-------'
      ENDIF ! asciiopflag

!------------------------------------------------------
! --- read and reproduce environmental inputs ---
! --- TO DO: convert NetCDF version of keyfile inputs 
! ---        into output DATA 
!------------------------------------------------------

!== READ NETCDF INITIALIZATION VARIABLES

      CALL eznc_get_0Dint(ncid_in,"autoox_fg",autoox_fg)
      CALL eznc_get_0Dint(ncid_in,"depos_fg",depos_fg)
      CALL eznc_get_0Dint(ncid_in,"dimer_fg",dimer_fg)
      CALL eznc_get_0Dint(ncid_in,"dyn_fg",dyn_fg)
      CALL eznc_get_0Dint(ncid_in,"flgdhf",flgdhf)
      CALL eznc_get_0Dint(ncid_in,"high_NOxfg",high_NOxfg)
      CALL eznc_get_0Dint(ncid_in,"zero_NOxfg",zero_NOxfg)
      CALL eznc_get_0Dint(ncid_in,"isomer_fg",isomer_fg)
      CALL eznc_get_0Dint(ncid_in,"OFR_fg",OFR_fg)
      CALL eznc_get_0Dint(ncid_in,"pvap_fg",pvap_fg)
      CALL eznc_get_0Dint(ncid_in,"ro2_fg",ro2_fg)
      CALL eznc_get_0Dint(ncid_in,"seedtyp_fg",seedtyp_fg)
      CALL eznc_get_0Dint(ncid_in,"soa_fg",soa_fg)
      CALL eznc_get_0Dint(ncid_in,"wall_fg",wall_fg)

      CALL eznc_get_0Dreal(ncid_in,"sumc",NDEN)
      CALL eznc_get_0Dreal(ncid_in,"tlen",TLEN)

      IF (OFR_fg.GT.0)THEN
        CALL eznc_get_0Dreal(ncid_in,"f185",f185)
        CALL eznc_get_0Dreal(ncid_in,"f254",f254)
      ENDIF
      IF (soa_fg.GT.0)THEN
        CALL eznc_get_0Dreal(ncid_in,"gamm",GAMM)
!        CALL eznc_get_0Dreal(ncid_in,"NVMW",NVMW)
        CALL eznc_get_0Dreal(ncid_in,"Mp",NVMW)
!        CALL eznc_get_0Dreal(ncid_in,"NVRO",NVRO)
        CALL eznc_get_0Dreal(ncid_in,"Rpo",NVRO)
!        CALL eznc_get_0Dreal(ncid_in,"SEED",SEED)
        CALL eznc_get_0Dreal(ncid_in,"cnv",SEED)
      ENDIF
      IF (wall_fg.GT.0)THEN
        !CALL eznc_get_0Dint(ncid_in,"ICHAM",ICHAM)
        CALL eznc_get_0Dint(ncid_in,"icham",ICHAM)
      ENDIF
      IF (pvap_scheme.EQ."SIM")THEN
        CALL eznc_get_2Dreal(ncid_in,"bk",bk,1,mxsim,1,4)
      ENDIF


!== read ascii indat.key for transfer to ascii out

      IF(asciiopflag) THEN
      OPEN(uin,file='indat.key')
        DO WHILE(.TRUE.)
          READ(uin,'(a100)',end=999) line
!          IF(INDEX(line,'/').NE.1.AND.INDEX(line,'END').NE.1) THEN

          IF(INDEX(line,'/').NE.1.AND.
     &       INDEX(line,'END').NE.1.AND.
     &       INDEX(line,'DATA').NE.1) THEN

            PRINT*,line(1:LEN_TRIM(line))
            WRITE(uou,*)line(1:LEN_TRIM(line))

          ENDIF
        ENDDO
999   CLOSE(uin)
      ENDIF

!--- --- --- --- --- --- --- --- --- --- --- ---
! DEBUG
!      CALL close_ncfile(ncid_out)
!      PRINT*,"DEBUG 1"
!      STOP
!------------------------------------------------------
! write KGWA as if it were an initialization line
! NetCDF write comes just below

      IF(ICHAM.EQ.2)THEN
        KGWA = 1.667E-03 ! s-1
      ELSE IF (ICHAM.EQ.3) THEN
        KGWA = 1.000E-03 ! s-1
      ENDIF

      IF(asciiopflag)THEN
        IF(ICHAM.EQ.2.OR.ICHAM.EQ.3)THEN
          WRITE(uou,*)'KGWA',KGWA,' / rate of gas-wall transfer, s-1'
        ENDIF
        WRITE(uou,*)'NDEN',NDEN,' / atmos number density, molec/cc '
        WRITE(uou,*)'END'
      ENDIF

!------------------------------------------------------
! --- NETCDF: HANDLE I/O DATA ---
* ---------------------------------------------------
!== DEFINE NETCDF INITIALIZATION VARIABLES: OUTPUT

      !CALL eznc_def_0Dint   (ncid_out,"mixing_fg")
      !CALL eznc_def_0Dint   (ncid_out,"emis_fg")
      !CALL eznc_def_0Dreal  (ncid_out,"noem_fg")
      !CALL eznc_def_0Dint   (ncid_out,"tracer_fg")
      !CALL eznc_def_0Dint   (ncid_out,"print_steadystate_fg")

      CALL eznc_def_0Dint   (ncid_out,"autoox_fg")
      CALL eznc_def_localatt(ncid_out,"autoox_fg","title",
     &     "flag invoking autooxidation: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"depos_fg")
      CALL eznc_def_localatt(ncid_out,"depos_fg","title",
     &     "flag invoking deposition: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"dimer_fg")
      CALL eznc_def_localatt(ncid_out,"dimer_fg","title",
     &     "flag invoking dimerisation: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"dyn_fg")
      CALL eznc_def_localatt(ncid_out,"dyn_fg","title",
     &     "flag invoking dynamic gas/aerosol partitioning:"//
     &     " 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"flgdhf")
      CALL eznc_def_localatt(ncid_out,"flgdhf","title",
     &     "flag invoking DHF formation: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"high_NOxfg")
      CALL eznc_def_localatt(ncid_out,"high_NOxfg","title",
     &     "flag invoking high-NOx chem only 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"zero_NOxfg")
      CALL eznc_def_localatt(ncid_out,"zero_NOxfg","title",
     &     "flag invoking zero-NOx chem only 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"isomer_fg")
      CALL eznc_def_localatt(ncid_out,"isomer_fg","title",
     &     "flag invoking isomerisation: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"OFR_fg")
      CALL eznc_def_localatt(ncid_out,"OFR_fg","title",
     &     "flag invoking OFR chemistry: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"pvap_fg")
      CALL eznc_def_localatt(ncid_out,"pvap_fg","title",
     &     "flag selecting vapor pressure scheme for "//
     &     "gas-aer exchange: 1/2 = Nannoolal/SIMPOL")
      CALL eznc_def_0Dint   (ncid_out,"ro2_fg")
      CALL eznc_def_localatt(ncid_out,"ro2_fg","title",
     &     "flag allowing RO2+RO2 chemistry: 0/1 = off/on")
      CALL eznc_def_0Dint   (ncid_out,"seedtyp_fg")
      CALL eznc_def_localatt(ncid_out,"seedtyp_fg","title",
     &     "flag selecting seed type: 0/1 = organic/inorganic ")
      CALL eznc_def_0Dint   (ncid_out,"soa_fg")
      CALL eznc_def_localatt(ncid_out,"soa_fg","title",
     &     "flag invoking SOA formation: 0/1/2 = off/equilib/dynamic")
      CALL eznc_def_0Dint   (ncid_out,"wall_fg")
      CALL eznc_def_localatt(ncid_out,"wall_fg","title",
     &     "flag invoking gas/wall exchange: 0/1 = off/on")

      CALL eznc_def_0Dreal  (ncid_out,"NDEN")
      CALL eznc_def_localatt(ncid_out,"NDEN","title",
     &     "atmospheric number density")
      CALL eznc_def_localatt(ncid_out,"NDEN","units","molecule.cm-3")
      CALL eznc_def_0Dreal  (ncid_out,"TLEN")
      CALL eznc_def_localatt(ncid_out,"TLEN","title",
     &     "model timestep")
      CALL eznc_def_localatt(ncid_out,"TLEN","units","s")

      IF (OFR_fg.GT.0)THEN
        CALL eznc_def_0Dreal  (ncid_out,"f185")
        CALL eznc_def_localatt(ncid_out,"f185","title",
     &       "photon flux at 185nm")
        CALL eznc_def_localatt(ncid_out,"f185","units",
     &       "photons.cm-3.s-1")
        CALL eznc_def_0Dreal  (ncid_out,"f254")
        CALL eznc_def_localatt(ncid_out,"f254","title",
     &       "photon flux at 254nm")
        CALL eznc_def_localatt(ncid_out,"f254","units",
     &       "photons.cm-3.s-1")
        CALL eznc_def_0Dreal  (ncid_out,"WATR")
        CALL eznc_def_localatt(ncid_out,"WATR","title",
     &       "water concentration")
        CALL eznc_def_localatt(ncid_out,"WATR","units",
     &       "molec.cm-3.s-1")
      ENDIF
      IF (soa_fg.GT.0)THEN
        CALL eznc_def_0Dreal  (ncid_out,"GAMM")
        CALL eznc_def_localatt(ncid_out,"GAMM","title",
     &       "gamma: organic aerosol bulk activity coefficient")
        CALL eznc_def_localatt(ncid_out,"GAMM","units","none")
        CALL eznc_def_0Dreal  (ncid_out,"SEED")
        CALL eznc_def_localatt(ncid_out,"SEED","title",
     &       "seed particle molecular concentration")
        CALL eznc_def_localatt(ncid_out,"SEED","units","molec cm-3")
        CALL eznc_def_0Dreal  (ncid_out,"NVMW")
        CALL eznc_def_localatt(ncid_out,"NVMW","title",
     &       "seed aerosol molecular weight")
        CALL eznc_def_localatt(ncid_out,"NVMW","units","amu")
        CALL eznc_def_0Dreal  (ncid_out,"NVRO")
        CALL eznc_def_localatt(ncid_out,"NVRO","title",
     &       "seed particle radius")
      ENDIF

      IF (wall_fg.GT.0)THEN
        CALL eznc_def_0Dint   (ncid_out,"ICHAM")
        CALL eznc_def_localatt(ncid_out,"ICHAM","title",
     &     "flag selecting chamber: 0/1/2/3 = none/Matsunaga-Ziemann"//
     &     "Ziemann(CU)/Jimenez(CU)")
!        CALL eznc_def_localatt(ncid_out,"KGWA","units","s-1")
!        CALL eznc_def_0Dreal  (ncid_out,"KGWA")
!        CALL eznc_def_localatt(ncid_out,"KGWA","title",
!     &       "gas-wall transfer rate")
      ENDIF
* ---------------------------------------------
! --- NetCDF: define output data arrays ---
! ---  names & dimensions, then units  ---
* ---------------------------------------------
!------------------------------------------------------
! --- NETCDF: DEFINE OUTPUT DATA ---
* ---------------------------------------------------

      CALL eznc_get_0Dint(ncid_in,"numsp",numsp)
      CALL eznc_def_dim(ncid_out,"numsp",numsp)      

* -----------------------------
! --- NetCDF: AUXILIARY DATA --
* -----------------------------
      
      CALL eznc_def_1Dreal(ncid_out,"time","nrt")
      CALL eznc_def_1Dreal(ncid_out,"temperature","nrt")
      CALL eznc_def_1Dreal(ncid_out,"humidity","nrt")
      CALL eznc_def_1Dreal(ncid_out,"OHexpos","nrt")
      CALL eznc_def_1Dreal(ncid_out,"O3expos","nrt")
      CALL eznc_def_1Dreal(ncid_out,"NO3expos","nrt")

      CALL eznc_def_localatt(ncid_out,"time","units","seconds")
      CALL eznc_def_localatt(ncid_out,"temperature","units","Kelvin")
      CALL eznc_def_localatt(ncid_out,"humidity","units","%")
      CALL eznc_def_localatt(ncid_out,"OHexpos","units","molec cm-3 s")
      CALL eznc_def_localatt(ncid_out,"O3expos","units","molec cm-3 s")
      CALL eznc_def_localatt(ncid_out,"NO3expos","units","molec cm-3 s")
! ---
      CALL eznc_def_1Dreal(ncid_out,"PERO1","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO2","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO3","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO4","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO5","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO6","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO7","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO8","nrt")
      CALL eznc_def_1Dreal(ncid_out,"PERO9","nrt")
      CALL eznc_def_1Dreal(ncid_out,"MEPERO","nrt")

      CALL eznc_def_localatt(ncid_out,"PERO1","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO2","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO3","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO4","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO5","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO6","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO7","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO8","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"PERO9","units","molec cm-3")
      CALL eznc_def_localatt(ncid_out,"MEPERO","units","molec cm-3")
! ---
      CALL eznc_def_1Dreal(ncid_out,"gas_phz_mass","nrt")
      CALL eznc_def_1Dreal(ncid_out,"aerosol_mass","nrt")
      CALL eznc_def_1Dreal(ncid_out,"wall_phz_mass","nrt")

      CALL eznc_def_localatt(ncid_out,"gas_phz_mass","units","ug m-3")
      CALL eznc_def_localatt(ncid_out,"aerosol_mass","units","ug m-3")
      CALL eznc_def_localatt(ncid_out,"wall_phz_mass","units","ug m-3")
! ---
      CALL eznc_def_1Dreal(ncid_out,"oa+seed_conc","nrt")
      CALL eznc_def_1Dreal(ncid_out,"org_wall_conc","nrt")
      CALL eznc_def_1Dreal(ncid_out,"Rp","nrt")
      CALL eznc_def_1Dreal(ncid_out,"oa+seed_molwt","nrt")

      CALL eznc_def_localatt(ncid_out,"oa+seed_conc","units",
     &                                               "molec cm-3")
      CALL eznc_def_localatt(ncid_out,"org_wall_conc","units",
     &                                                "molec cm-3")
      CALL eznc_def_localatt(ncid_out,"Rp","units","cm")
      CALL eznc_def_localatt(ncid_out,"oa+seed_molwt","units",
     &                                                "g mole-1")
! ---
      CALL eznc_def_1Dreal(ncid_out,"O_to_C_gas","nrt")
      CALL eznc_def_1Dreal(ncid_out,"O_to_C_aer","nrt")
      CALL eznc_def_1Dreal(ncid_out,"O_to_C_wall","nrt")
      CALL eznc_def_1Dreal(ncid_out,"H_to_C_gas","nrt")
      CALL eznc_def_1Dreal(ncid_out,"H_to_C_aer","nrt")
      CALL eznc_def_1Dreal(ncid_out,"H_to_C_wall","nrt")
      CALL eznc_def_1Dreal(ncid_out,"N_to_C_gas","nrt")
      CALL eznc_def_1Dreal(ncid_out,"N_to_C_aer","nrt")
      CALL eznc_def_1Dreal(ncid_out,"N_to_C_wall","nrt")

      CALL eznc_def_localatt(ncid_out,"O_to_C_gas","units","ratio")
      CALL eznc_def_localatt(ncid_out,"O_to_C_aer","units","ratio")
      CALL eznc_def_localatt(ncid_out,"O_to_C_wall","units","ratio")
      CALL eznc_def_localatt(ncid_out,"H_to_C_gas","units","ratio")
      CALL eznc_def_localatt(ncid_out,"H_to_C_aer","units","ratio")
      CALL eznc_def_localatt(ncid_out,"H_to_C_wall","units","ratio")
      CALL eznc_def_localatt(ncid_out,"N_to_C_gas","units","ratio")
      CALL eznc_def_localatt(ncid_out,"N_to_C_aer","units","ratio")
      CALL eznc_def_localatt(ncid_out,"N_to_C_wall","units","ratio")
! ---
      CALL eznc_def_1Dreal(ncid_out,"jNO2","nrt")
      CALL eznc_def_1Dreal(ncid_out,"jNO3","nrt")
      CALL eznc_def_1Dreal(ncid_out,"jO3","nrt")
      CALL eznc_def_1Dreal(ncid_out,"jH2O2","nrt")
      CALL eznc_def_1Dreal(ncid_out,"jHONO","nrt")
      CALL eznc_def_1Dreal(ncid_out,"jCH2O","nrt")

      CALL eznc_def_localatt(ncid_out,"jNO2","units","s-1")
      CALL eznc_def_localatt(ncid_out,"jNO3","units","s-1")
      CALL eznc_def_localatt(ncid_out,"jO3","units","s-1")
      CALL eznc_def_localatt(ncid_out,"jH2O2","units","s-1")
      CALL eznc_def_localatt(ncid_out,"jHONO","units","s-1")
      CALL eznc_def_localatt(ncid_out,"jCH2O","units","s-1")
! ---
      CALL eznc_def_1Dint(ncid_out,"idchrom","njchrom")
      CALL eznc_def_localatt(ncid_out,"idchrom","description",
     &        "photolysis id label")

      CALL eznc_def_2Dreal(ncid_out,"jchrom","nrt","njchrom")
      CALL eznc_def_localatt(ncid_out,"jchrom","units","s-1")

* -----------------------------
! --- NetCDF: SPECIES DATA --
* -----------------------------

      CALL eznc_def_1Dchar(ncid_out,"name",'lco','numsp')
      CALL eznc_def_localatt(ncid_out,"name","description",
     &            '6-char molecular identifier. UNIQUE TO MECH.')

      CALL eznc_def_1Dchar(ncid_out,"molec_formula","lmf","numsp")
      CALL eznc_def_localatt(ncid_out,"molec_formula","description",
     &            'CnHwNxOySz')

      CALL eznc_def_1Dchar(ncid_out,"SMILES","lfo","numsp")
      CALL eznc_def_localatt(ncid_out,"SMILES","description",
     &      'formula in SMILES format, nonradicals only (not-USMILES)')

      CALL eznc_def_1Dchar(ncid_out,"chemical_formula","lfo","numsp")
      CALL eznc_def_localatt(ncid_out,"chemical_formula","description",
     &            'chemical formula, written by GECKO-A')

      CALL eznc_def_1Dreal(ncid_out,"MW","numsp")
      CALL eznc_def_localatt(ncid_out,"MW","units","a.m.u.")
      CALL eznc_def_localatt(ncid_out,"MW","description",
     &            'molecular weight')

      IF(radicals) THEN
        CALL eznc_def_1Dint(ncid_out,"rad","numsp")
        CALL eznc_def_1Dint(ncid_out,"RO2","numsp")
      ENDIF
      CALL eznc_def_1Dint(ncid_out,"ifg","numsp")
      CALL eznc_def_1Dint(ncid_out,"phz","numsp")

      CALL eznc_def_2Dint(ncid_out,"fungrp","nfgrp","numsp")
      CALL eznc_def_localatt(ncid_out,"fungrp","description",
     &            'number of functional groups of each type')
      CALL eznc_def_localatt(ncid_out,"fungrp","names",
     &            'F_A,F_D,F_E,F_EE,F_G,F_H,F_K,F_N,'//
     &            'F_P,F_R,F_T,F_U,F_V,F_X')
      CALL eznc_def_localatt(ncid_out,"fungrp","dictionary",
     &     'Acid, alDehyde, Ether, pEroxidE, peroxy-acid, '//
     &     'Hydroperoxy, Ketone, Nitrate, hydrOxy, PAN, '//
     &     'aRomatic, aliphaTic ring, Unsaturated, nitrite, ketene')

      CALL eznc_def_1Dreal(ncid_out,"Cstar_298","numsp")
      CALL eznc_def_localatt(ncid_out,"Cstar_298","units","ug m-3")
      IF(pvap_scheme.EQ."NAN")THEN
        CALL eznc_def_localatt(ncid_out,"Cstar_298","description",
     $              'NANNOOLAL saturation concentration @ 298K')
        CALL eznc_def_localatt(ncid_out,"Cstar_298","reference",
     $ 'Nannoolal et al (2004) doi:10.1016/j.fluid.2004.09.001 &'//
     $ ' (2008) doi:10.1016/j.fluid.2008.04.020')
      ELSE
        CALL eznc_def_localatt(ncid_out,"Cstar_298","description",
     $              'SIMPOL saturation concentration @ 298K')
        CALL eznc_def_localatt(ncid_out,"Cstar_298","reference",
     $              'Pankow & Asher (2008) doi:10.5194/acp-8-2773-2008')
      ENDIF
      CALL eznc_def_localatt(ncid_out,"Cstar_298","equation",
     & 'Cstar_298 = Psat_298(atm) * MW(ug mol-1) * '//
     & '/ [ R(atm L K-1 mol-1) * 298(K) ] * 1e+9')

      CALL eznc_def_1Dreal(ncid_out,"Tb","numsp")
      CALL eznc_def_localatt(ncid_out,"Tb","units","K")
      CALL eznc_def_localatt(ncid_out,"Tb","description",
     & 'boiling point =>  Trb = T(K) / Tb : Nannoolal(2004,2008)')

      CALL eznc_def_1Dreal(ncid_out,"dB","numsp")
      CALL eznc_def_localatt(ncid_out,"dB","units","none")
      CALL eznc_def_localatt(ncid_out,"dB","description",
     & 'param containing group contribs & # of atoms: Nannoolal(04,08)')
      CALL eznc_def_localatt(ncid_out,"dB","equation",
     & 'Psat(atm) = e^( log(10)*(4.1012 + dB) *(Trb-1)/(Trb-(1/8)) )')

      CALL eznc_def_1Dreal(ncid_out,"DVa","numsp")
      CALL eznc_def_localatt(ncid_out,"DVa","units","none")
      CALL eznc_def_localatt(ncid_out,"DVa","description",
     & 'diffusion volume (dimensionless) per Reid(1987)')
      CALL eznc_def_localatt(ncid_out,"DVa","reference",
     & ' Table 11-1: Reid, Prausnitz & Poling (1987),'//
     & ' The properties of gases and liquids, McGraw Hill, New York')

      CALL eznc_def_localatt(ncid_out,"DVa","equation",
     & 'Dg(cm2s-1) = 0.00143 T^(7/8)/'//
     & '(mAB^(1/2)*(DVa^(1/3)+DVb^(1/3))*P(atm)) '//
     & 'where  mAB = reduced mass = 2/(1/MW+1/mwAir)(g.mol-1) '//
     & 'and DVb = diffusion volume of air = 9.746')
      CALL eznc_def_localatt(ncid_out,"DVa","references",
     & 'Tang, ACP (2015), Kulmala & Wagner, JAS (2001), '//
     & 'Seinfeld & Pandis pp 537-577 (2006)')

      CALL eznc_def_0Dreal(ncid_out,"KGWA")
      CALL eznc_def_localatt(ncid_out,"KGWA","units","s-1")
      CALL eznc_def_localatt(ncid_out,"KGWA","description",
     & 'chamber-specific gas-wall transfer rate')

      CALL eznc_def_1Dreal(ncid_out,"kwg","numsp")
      CALL eznc_def_localatt(ncid_out,"kwg","description",
     & 'chamber-&-species-specific wall-gas transfer rate')

      CALL eznc_def_1Dreal(ncid_out,"kHenry","numsp")
      CALL eznc_def_localatt(ncid_out,"kHenry","units",'M.atm-1')
      CALL eznc_def_localatt(ncid_out,"kHenry","description",
     & 'Henrys Law const at pH=7 & 298K')
      CALL eznc_def_localatt(ncid_out,"kHenry","reference",
     & 'Raventos-Duran, ACPD, 2010')

      CALL eznc_def_1Dreal(ncid_out,"dH2Odx","numsp")
      CALL eznc_def_1Dreal(ncid_out,"reacf","numsp")

      CALL eznc_def_2Dreal(ncid_out,"concentration","nrt","numsp")
      CALL eznc_def_localatt(ncid_out,"concentration","units",
     &                                                "molec.cm-3")

      CALL switch_ncfile_to_data_mode(ncid_out)


      WRITE(uou,*)'---------DICTIONARY OF MOLECULAR PARAMETERS---------'

      WRITE(uou,*)' STRINGS : '

      WRITE(uou,*)'name = 6-char molecular identifier. UNIQUE TO MECH.'
      WRITE(uou,*)'molec_formula = CnHwNxOySz'
      WRITE(uou,*)'SMILES = formula in SMILES format, nonradicals only'
      WRITE(uou,*)'         (nb: not "USMILES" format)'
      WRITE(uou,*)'chem_formula = chemical formula, written by GECKO-A'
      WRITE(uou,*)' VALUES : '
      WRITE(uou,*)'MW = molecular weight (a.m.u.)'
      WRITE(uou,*)'Cstar_298 = saturation concentration @ 298K (ug m-3)'
      IF(pvap_scheme.EQ."NAN")THEN
      WRITE(uou,*)'           Psat calculated per Nannoolal (2004,2008)'
      ELSE
      WRITE(uou,*)'    SIMPOL Psat calculated per Pankow & Asher (2008)'
      ENDIF
      WRITE(uou,*)'      => Cstar_298 = Psat_298(atm) * MW(ug mol-1) * '
      WRITE(uou,*)'            / [ R(atm L K-1 mol-1) * 298(K) ] * 1e+9'
      WRITE(uou,*)'Saturation vapor pressure per Nannoolal (2004, 2008)'
      WRITE(uou,*)'  Tb = boiling point (K)  =>  Trb = T(K) / Tb '
      WRITE(uou,*)'  dB = param containing group contribs & # of atoms:'
      WRITE(uou,*)'     => Psat(atm) = e^( log(10)*(4.1012 + dB) '
      WRITE(uou,*)'                         *(Trb-1)/(Trb-(1/8)) )'
      WRITE(uou,*)'Gas-Particle partitioning rate parameters, refs:'
      WRITE(uou,*)'Tang, ACP (2015), Kulmala & Wagner, JAS (2001), '
      WRITE(uou,*)'Seinfeld & Pandis pp 537-577 (2006)'
      WRITE(uou,*)'  DVa = diffusion volume, species A (dimensionless) '
      WRITE(uou,*)'   per Reid(1987): USE for diffusion coefficient Dg '
      WRITE(uou,*)'   for trace gas A in bath gas B (air): Fuller(1966)'
      WRITE(uou,*)'  > mAB = reduced mass = 2/(1/MW+1/mwAir)(g.mol-1)  '
      WRITE(uou,*)'  > DVb = diffusion volume of air = 9.746 '
      WRITE(uou,*)'  => Dg (cm2s-1) = .00143 T^(7/8) / '
      WRITE(uou,*)'         (mAB^(1/2)*(DVa^(1/3) + DVb^(1/3))*P(atm)) '
! TO DO: include all thr following info in NetCDF file 
      WRITE(uou,*)'  AND:'
      WRITE(uou,*)'  > NVMW, SEED, NVRO are supplied in Keyfile  '
      WRITE(uou,*)'  > RHOP = seed density = 0.9 (org), 1.77 (inorg)   '
      WRITE(uou,*)'  => Cp = particle number = '
      WRITE(uou,*)'      (NVMW*SEED)/(Navo*(4./3.)*Pi*RHOP*(NVRO**3.)) '
      WRITE(uou,*)'  AND:'
      WRITE(uou,*)'  > Rp = total particle radius (cm) (see AUX DATA)'
      WRITE(uou,*)'  AND:'
      WRITE(uou,*)'  > R = gas constant = 8.314e-7 g.cm2.s-2.K-1.mol-1 '
      WRITE(uou,*)'  > ci = mean velocity =(8*R*T/(Pi*MW))^0.5 (cm.s-1)'
      WRITE(uou,*)'  => Kn = Knudson number = 3*Dg/Rp/ci  (molec-1)'
      WRITE(uou,*)'  => FS = Fuchs-Sutugin parameter = (Kn+1) / '
      WRITE(uou,*)'       (0.377*Kn + 1 + (4/3)*alpha^(-1)*(Kn + Kn^2))'
      WRITE(uou,*)'  THUS:'
      WRITE(uou,*)'  => instantaneous Gas-Aer partitioning rate '
      WRITE(uou,*)'  => =>  kga = 4*pi*Dg*Cp*Rp*FS (s-1) '

      WRITE(uou,*)'kwg = wall-gas transfer rate'
      WRITE(uou,*)'                  (chamber-&-species-specific)'
      WRITE(uou,*)'Henrys Law & Dry Deposition params per Wesely(1989):'
      WRITE(uou,*)'          For equations see docs at http://cires1.'
      WRITE(uou,*)'          colorado.edu/jimenez-group/software/GECKO/'
      WRITE(uou,*)'  kHenry = Henrys Law const (M.atm-1) at pH=7 & 298K'
      WRITE(uou,*)'             assessed per Raventos-Duran, ACPD, 2010'
      WRITE(uou,*)'  dH2Odx = ratio of diffusion coefficients D_H2O/D_x'
      WRITE(uou,*)'  reacf  = reactivity factor (= 0, 0.1, 1)'

      IF(fgcodes)THEN
      WRITE(uou,*)'--------DICTIONARY OF FUNCTIONAL GROUP CODES--------'
      WRITE(uou,*)'F_A = acid,        F_D = aldehyde, F_E = ether,'
      WRITE(uou,*)'F_EE = peroxide,   F_G = peroxy-acid, '
      WRITE(uou,*)'F_H = hydroperoxy, F_K = ketone,   F_N = nitrate,'
      WRITE(uou,*)'F_O = hydroxy,     F_P = PAN,      F_R = aromatic,'
      WRITE(uou,*)'F_T = aliphatic ring,             F_U = unsaturated,'
      WRITE(uou,*)'F_V = nitrite,     F_X = ketene'
      ENDIF

      WRITE(uou,*)'------------DICTIONARY OF TYPE CODES----------------'

      IF(radicals) THEN
        WRITE(uou,*)'rad = radical type: '
        WRITE(uou,*)'  1 = RO; 2 = alkylRO2; 3 = acylRO2 ; 4 = Criegee '
        WRITE(uou,*)'RO2 = RO2 type:                                   '
        WRITE(uou,*)'  1 = linear primary ; '
        WRITE(uou,*)'  2 = branched primary RO2 ; '
        WRITE(uou,*)'  3 = a- or b- O-substituted primary RO2 ;        '
        WRITE(uou,*)'  4 = CH3CH(OO.)CH3 ;  '
        WRITE(uou,*)'  5 = secondary RO2 (C>3) ;  '
        WRITE(uou,*)'  6 = a- or b- O-substituted secondary RO2 ;      '
        WRITE(uou,*)'  7 = Tertiary RO2 ;      '
        WRITE(uou,*)'  8 = a- or b- O-substituted tertiary RO2 ;       '
        WRITE(uou,*)'  9 = acyl peroxy                  '
      ENDIF

      WRITE(uou,*)'ifg = inorganic flag: '
      WRITE(uou,*) '  1=inorganic; 0 = organic'

      WRITE(uou,*)'phz = phase: '
      WRITE(uou,*)'  1 = gas; 2 = particle; 3 = wall'

* ----------------------------------
* read dictionary (exclude keywords)
* ----------------------------------
      write(6,*) "reading dictionary"

      CALL eznc_get_dimension(ncid_in,"mxdic",ndic)
      CALL eznc_get_1Dchar(ncid_in,"dicnam",name,lco,1,ndic)
      CALL eznc_get_1Dchar(ncid_in,"chem",chem,lfo,1,ndic)
      CALL eznc_get_1Dchar(ncid_in,"code",code,lfl,1,ndic)

      PRINT*,name(ndic),chem(ndic),code(ndic)

      WRITE(6,*) 'number of species in the dictionary=',ndic
      WRITE(6,*) 'maxsp =',maxsp
      IF(maxsp.LT.ndic) STOP

* ----------------------------------------------------------
* read Nannoolal coefficients
* ----------------------------------------------------------
      write(6,*) "reading pvap params"

      CALL eznc_get_0Dint(ncid_in,"nsat",nsat)
      PRINT*,"nsat = ",nsat

      CALL eznc_get_2Dreal(ncid_in,"nandat",Tb, 1,nsat,1,1)
      CALL eznc_get_2Dreal(ncid_in,"nandat",dB, 1,nsat,2,2)

      CALL eznc_get_2Dreal(ncid_in,"simpgroup",simpgroup,
     &                                          1,nsat,1,mxsim)

* ----------------------------------------------------------
* read Henry coefficients
* ----------------------------------------------------------
      write(6,*) "reading Henry params"

      CALL eznc_get_0Dint(ncid_in,"ndep",nhenry)
      CALL eznc_get_2Dreal(ncid_in,"depdat",dh2odx, 1,nsat,1,1)
      CALL eznc_get_2Dreal(ncid_in,"depdat",kHenry, 1,nsat,2,2)
      CALL eznc_get_2Dreal(ncid_in,"depdat",reacf, 1,nsat,2,2)

      PRINT*,"nhenry = ",nhenry

* --------------------------------
* read data from the results file
* --------------------------------
      cout = 0.
      lensp = maxlsp
      WRITE(6,*) " numsp = ",numsp

! READ SPECIES NAMES ETC FROM NETCDF INPUT FILE

      CALL eznc_get_0Dint(ncid_in,"numsp",numsp)
      CALL eznc_get_1Dchar(ncid_in,"chrsp",chrsp,maxlsp,1,numsp)
      CALL eznc_get_1Dint(ncid_in,"satid",satid,1,numsp)
      !CALL eznc_get_1Dint(ncid_in,"nanid",satid,1,numsp)
      
! Wmol is calculated more accurately below
      CALL eznc_get_1Dreal(ncid_in,"wmol",wmol,1,numsp)
      WRITE(6,*) chrsp(1), (chrsp(i),i=numsp-3,numsp)

! READIN RO2 FROM NETCDF FILE 
      CALL eznc_get_2Dint(ncid_in,"idchemro2",idchemro2,1,mxro2cl,1,9)
      DO k=1,9
        DO i=1,mxro2cl
          IF(idchemro2(i,k).EQ.0)THEN
            n_ro2(k) = i-1
            EXIT ! jump to next 1
          ENDIF
          chrsp_ro2(i,k)(1:7)=chrsp(idchemro2(i,k))(1:7)
        ENDDO
      ENDDO

! --- find specific species indices ---
      DO i=1,numsp 
        IF (INDEX(name(i),'O3 ').EQ.1) idO3 = i
        IF (INDEX(name(i),'HO ').EQ.1) idOH = i
        IF (INDEX(name(i),'NO3 ').EQ.1) idNO3 = i
      ENDDO
      WRITE(6,*) "idO3,OH,NO3 = ",idO3,idOH,idNO3

* -------------------------------------------------------------
* read data: keep only data corresponding to reftimes
* -------------------------------------------------------------

      ndat=0
      nout=0
      kt = 1
       
30    CONTINUE !DO ndat = 1,mdat

      ndat = ndat + 1
      IF(ndat.GT.mdat) GOTO 50

      CALL eznc_get_1Dreal(ncid_in,"time",time1,ndat,ndat)
      time_read = time1(1)
      PRINT*,"ndat, time_read:",ndat,time_read

! testing & choosing
    
      IF(lasttime) THEN
          nout = 1
          idat(nout)=ndat
          PRINT*,"lasttime option not working yet"
           STOP
      ELSEIF(alltimes) THEN
          PRINT*,"alltimes option not written yet"
           STOP
      ELSE

        DO it=kt,nrt

          PRINT*,ndat,time_read,reftime(it)
! following line only works for timesteps => 1s!
          IF(opstep.GE.1.)THEN
            IF (NINT(time_read).ge.reftime(it)) THEN
                nout = nout + 1
                kt = it+1
                EXIT
              ELSE
                GO TO 30
              ENDIF ! (time.lt.reftime(it)) CYCLE
            ELSE ! opstep < 1s
                 ! subtract (opstep/10.) from reftime to avoid precision
                 ! issues)
              IF ((time_read).ge.(reftime(it)-opstep/10.)) THEN
                nout = nout + 1
                kt = it+1
                EXIT
              ELSE
                GO TO 30
              ENDIF ! (time.lt.reftime(it)) CYCLE
            ENDIF ! opstep > 1s
          ENDDO ! it=1,nrt
        ENDIF ! (lasttime, alltimes)
   
!-----------------------
!--reading
!-----------------------
! don't interpolate to desired time: assign ACTUAL time read.
        time(nout) = time_read
        PRINT*,ndat,time_read, time(nout),nout

        CALL eznc_get_2Dreal(ncid_in,"temp",temp2,1,1,ndat,ndat)
                                          temp(nout) = temp2(1,1)
        CALL eznc_get_2Dreal(ncid_in,"rh",rhum2,1,1,ndat,ndat)
                                          rhum(nout) = rhum2(1,1)

        CALL eznc_get_3Dreal(ncid_in,"conc",cgas3,
     &                                      1,numsp,1,1,ndat,ndat)
                 cout(nout,1:numsp) = cgas3(1:numsp,1,1)

        CALL eznc_get_2Dreal(ncid_in,"jchrom",jchrom,
     &                                       1,mjchrom,ndat,ndat) 
                 ojchrom(nout,:) = jchrom(:,1)

!------------------------------------------------------
! --- read soa parameter output and extract relevant times
! -- NB::: Rp,mwaer ONLY AVAILABLE in current NetCDF model output
! ---      IF soa_fg = 2

        IF(soa_fg.EQ.2) THEN
          CALL eznc_get_2Dreal(ncid_in,"Rp",Rp,1,1,ndat,ndat)
          CALL eznc_get_2Dreal(ncid_in,"Mp",Mp,1,1,ndat,ndat)
          CALL eznc_get_2Dreal(ncid_in,"mwaer",mwaer,1,1,ndat,ndat)
          CALL eznc_get_2Dreal(ncid_in,"ctotaer",ctotaer,1,1,ndat,ndat)
        ELSE
!          PRINT*,"BYPASSING READ OF SOA OUT PARAMS"
          GOTO 997
        ENDIF

997     CONTINUE

!---------------------------------
! manage time selection for read
!---------------------------------

        IF (ndat.GT.mdat) THEN
          PRINT*,ndat,mdat
          WRITE(6,*) '--error--, number of data exceed mdat'
          STOP
        ENDIF

        IF(kt.EQ.nrt+1) GOTO 50
        GOTO 30
50      CONTINUE

!---------------------------------
!--- we should now have read all the required data ---
!---------------------------------

        ndat=ndat-1
        WRITE(6,*) 'number of data read from the '//extn//
     &             ' result file =',ndat
        WRITE(6,*) 'number of output data requested =',nrt
        WRITE(6,*) 'number of data to be output =',nout

* -----------------------------
* EXTRACT AND COMPUTE DATA NEEDED
* -----------------------------

!--------------------------------------------
! cumulative O3,OH,NO3 exposure, based on mean over each timestep
!--------------------------------------------
        DO i = 2,nrt
          dt =  time(i)-time(i-1)
          OHexp(i) = OHexp(i-1)  + dt*(cout(i,idOH)+cout(i-1,idOH))/2.
          O3exp(i) = O3exp(i-1)  + dt*(cout(i,idO3)+cout(i-1,idO3))/2.
          NO3exp(i)= NO3exp(i-1) + dt*(cout(i,idNO3)+cout(i-1,idNO3))/2.
        ENDDO

!------------------------------------------------------
* compute the number of C and function for the various species
* -------------------------------------------------------------
        memo=1
        jpvap = 1
        jhenry = 1
        ncarbon=0
        noxygen=0
        nnitrogen=0
        nhydrogen=0
        nsulfur=0

! --- loop species ---
        PRINT*,"first loop"

! DEBUG
!        PRINT*,"bypassing first loop"
!        GOTO 997

        i1 = ilo
        i2 = ihi
        IF(ilo.EQ.0) i1 = 1
        IF(ihi.EQ.0) i2 = numsp

        DO i=i1,i2

          IF (chrsp(i)(2:2).NE."1".AND.chrsp(i)(2:2).NE."2".AND.
     &        chrsp(i)(2:2).NE."3".AND.chrsp(i)(2:2).NE."4")THEN
!==Non-radicals...

! assign pvap params to each non-radical species
            DO j=jpvap,nsat
              IF (satid(i).EQ.j) THEN
                oTb(i) = Tb(j)
                odB(i) = dB(j)
                osimpgroup(i,:) = simpgroup(j,:)
! FIX FOR MISSING satid VALUES IN 'A' & 'W' PHASES
                IF(chrsp(i+1)(1:1).EQ."A".OR.
     &             chrsp(i+1)(1:1).EQ."W")THEN
                  oTb(i+1) = Tb(j)
                  odB(i+1) = dB(j)
                  osimpgroup(i+1,:) = simpgroup(j,:)
                ENDIF
                IF(chrsp(i+2)(1:1).EQ."A".OR.
     &             chrsp(i+2)(1:1).EQ."W")THEN
                  oTb(i+2) = Tb(j)
                  odB(i+2) = dB(j)
                  osimpgroup(i+2,:) = simpgroup(j,:)
                ENDIF
                jpvap = j
                EXIT
              ENDIF
            ENDDO! j=1,nsat

! assign kHenry to each non-radical species
            DO j=jhenry,nhenry
              IF (satid(i).EQ.j) THEN
                okHenry(i) = kHenry(j)
                odh2odx(i) = dh2odx(j)
                oreacf(i) = reacf(j)
! FIX FOR MISSING satid VALUES IN 'A' & 'W' PHASES
                IF(chrsp(i+1)(1:1).EQ."A".OR.
     &             chrsp(i+1)(1:1).EQ."W")THEN
                  okHenry(i+1) = kHenry(j)
                  odh2odx(i+1) = dh2odx(j)
                  oreacf(i+1) = reacf(j)
                ENDIF
                IF(chrsp(i+2)(1:1).EQ."A".OR.
     &             chrsp(i+2)(1:1).EQ."W")THEN
                  okHenry(i+2) = kHenry(j)
                  odh2odx(i+2) = dh2odx(j)
                  oreacf(i+2) = reacf(j)
                ENDIF
                jhenry = j
                EXIT
              ENDIF
            ENDDO! j=1,nhenry

          ELSE ! is radical
!==Radicals...

! assign ro2 type to each species
! FOR LATER: USE INDEXING ALREADY PRESENT IN NETCDF FILE
! (could use directly from idchemro2 instead of comparing chrsps)
          IF (chrsp(i)(2:2).EQ."2".OR.
     &        chrsp(i)(2:2).EQ."3")THEN
            DO k = 1,nro2typ
              DO j=1,n_ro2(k)
                IF (chrsp(i)(2:7).eq.chrsp_ro2(j,k)(2:7)) THEN
                  perotyp(i) = k
                  EXIT
                ENDIF
              ENDDO! j=1,nro2(k)
            ENDDO! k=1,9
          ENDIF ! ro2
         ENDIF ! non-radical

! assign ro2 type = 10 to CH3O2 (aka MEPERO)
         IF (chrsp(i)(2:6).EQ."CH3O2") perotyp(i) = 10

! --- assign species names/ formulae/ fgrp-/ phase-codes ---
! FOR LATER: USE INDEXING ALREADY PRESENT IN NETCDF FILE
! (may not need 'o' arrays)
         DO j=memo,ndic
           IF (chrsp(i)(2:7).EQ.name(j)) THEN
         
             PRINT*,i," 1st ",chrsp(i)

             oname(i)=name(j)
             ochem(i)=chem(j)
             ocode(i)=code(j)

             SELECT CASE(chrsp(i)(1:1))
               CASE('G')
                 iphz(i)=1
               CASE('A')
                 iphz(i)=2
               CASE('W')
                 iphz(i)=3
               CASE DEFAULT
                 iphz(i)=0
             END SELECT

! --- count atoms ---
!     (using "number" produces segfaults: avoid!)

            DO ipos=1,LEN_TRIM(chem(j))
              IF(chem(j)(ipos:ipos).eq.' ') EXIT
              IF((chem(j)(ipos:ipos).eq.'C') .and.
     &         (chem(j)(ipos:ipos+1).ne.'Cl')) ncarbon(i)=ncarbon(i)+1
              IF(chem(j)(ipos:ipos).eq.'c')    ncarbon(i)=ncarbon(i)+1
              IF(chem(j)(ipos:ipos).eq.'O')    noxygen(i)=noxygen(i)+1
              IF(chem(j)(ipos:ipos+1).eq.'O2') noxygen(i)=noxygen(i)+1
              IF(chem(j)(ipos:ipos+1).eq.'O3') noxygen(i)=noxygen(i)+2
              IF(chem(j)(ipos:ipos+1).eq.'O4') noxygen(i)=noxygen(i)+3
              IF(chem(j)(ipos:ipos+1).eq.'O5') noxygen(i)=noxygen(i)+4
              IF(chem(j)(ipos:ipos).eq.'N') nnitrogen(i)=nnitrogen(i)+1
              IF(chem(j)(ipos:ipos).eq.'S') nsulfur(i)=nsulfur(i)+1
              IF(chem(j)(ipos:ipos).eq.'s') nsulfur(i)=nsulfur(i)+1
              IF(chem(j)(ipos:ipos).eq.'H') nhydrogen(i)=nhydrogen(i)+1
           IF(chem(j)(ipos:ipos+1).eq.'H2') nhydrogen(i)=nhydrogen(i)+1
           IF(chem(j)(ipos:ipos+1).eq.'H3') nhydrogen(i)=nhydrogen(i)+2
           IF(chem(j)(ipos:ipos+1).eq.'H4') nhydrogen(i)=nhydrogen(i)+3
              ENDDO

! --- assess accurate molecular mass ---
! (midpoint between limits specified by NIST at:
! https://physics.nist.gov/cgi-bin/Compositions/stand_alone.pl?ele=&all=all&ascii=ascii&isotype=some

            wmol(i) = 12.0106*REAL(ncarbon(i)) + 
     &                15.9994*REAL(noxygen(i)) +
     &                32.0675*REAL(nsulfur(i)) +
     &                14.006855*REAL(nnitrogen(i)) +
     &                1.007975*REAL(nhydrogen(i))

! --- assess dimensionless diffusion volume---
! (non-radicals only)
          IF (chrsp(i)(2:2).NE."1".AND.chrsp(i)(2:2).NE."1".AND.
     &        chrsp(i)(2:2).NE."3".AND.chrsp(i)(2:2).NE."4")THEN

            DVa(i) = 15.9*REAL(ncarbon(i)) + 
     &               6.11*REAL(noxygen(i)) +
     &               4.54*REAL(nnitrogen(i)) +
     &               2.31*REAL(nhydrogen(i))
!    if chem = aromatic or heteroyclic ring, subtract 18.3
            IF ((INDEX(chem(j),'c').NE.0).OR.
     &          (INDEX(chem(j),'C1').NE.0).OR.
     &          (INDEX(chem(j),'O1').NE.0)) DVa(i) = Dva(i) - 18.3
            !WRITE(6,*)"DVa(i)",i,DVa(i)
          ENDIF ! org non-radical

! --- assess mass & elemental ratio time-series ---
! (will need to divide ratios by total conc in each phase once that is
! known)
          IF(ncarbon(i).GT.0.AND.nhydrogen(i).GT.0
     &         .AND.oname(i).NE.spignore)THEN

! DEBUG
!       PRINT*,"            elem ratios"

             DO k=1,nout
               SELECT CASE(iphz(i))
                 CASE(1)
                   ctotgas(k)  = ctotgas(k) +cout(k,i)
                   mgas_out(k) = mgas_out(k)+cout(k,i)
     &                                      *wmol(i)/Navo*1e12
                   OtoC_gas(k) = OtoC_gas(k)+REAL(noxygen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   HtoC_gas(k) = HtoC_gas(k)+REAL(nhydrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   NtoC_gas(k) = NtoC_gas(k)+REAL(nnitrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))

                 CASE(2)
                   corgaer(k)  = corgaer(k) +cout(k,i)
                   maer_out(k) = maer_out(k)+cout(k,i)
     &                                      *wmol(i)/Navo*1e12
                   OtoC_aer(k) = OtoC_aer(k)+REAL(noxygen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   HtoC_aer(k) = HtoC_aer(k)+REAL(nhydrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   NtoC_aer(k) = NtoC_aer(k)+REAL(nnitrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))

                 CASE(3)
                   corgwal(k)  = corgwal(k) +cout(k,i)
                   mwal_out(k) = mwal_out(k)+cout(k,i)
     &                                      *wmol(i)/Navo*1e12
                   OtoC_wal(k) = OtoC_wal(k)+REAL(noxygen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   HtoC_wal(k) = HtoC_wal(k)+REAL(nhydrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))
                   NtoC_wal(k) = NtoC_wal(k)+REAL(nnitrogen(i))
     &                            *cout(k,i)/REAL(ncarbon(i))

                END SELECT
             ENDDO
          ENDIF

! -- assess Cstar(i) at 298K and at current temperature -
! NANNOOLAL METHOD
!'Cstar_298 = saturation concentration @ 298K (ug m-3)'
!'          => Cstar_298 = Pvap(atm) * N(molec m-3) * ' 
!'             298/T(K) * MW(g/mol) / N_avo(mol-1) * 1e6 (ug/g)' 
 
! DEBUG
!       PRINT*,"              cstar"
          Cstar_298(i)=0.
          Cstar_T(i)=0.

          IF(oTb(i).GT.0)THEN 
            IF(pvap_scheme.EQ."NAN")THEN
! for T_current
              Trb = temp(nout)/oTb(i)
              pvap=EXP(LOG(10.)*(4.1012+odB(i))*(Trb-1.)/(Trb-(1./8.)))
              !Cstar_T(i)=pvap*wmol(i)/(Ratm*293) * 1.e+9
              ! for wall
              Cstar_T(i)=pvap*mw_wall/(Ratm*293) * 1.e+9
! for 298 K
              Trb = 298/oTb(i)
              pvap=EXP(LOG(10.)*(4.1012+odB(i))*(Trb-1.)/(Trb-(1./8.)))
              Cstar_298(i)=pvap*wmol(i)/(Ratm*298) * 1.e+9
            ELSE ! "SIM"
! for T_current
              log10psat = 0.
              DO k=1,mxsim
                bk_T = ((bk(k,1)/temp(nout))+ bk(k,2)+       
     &                  (bk(k,3)*temp(nout))+(bk(k,4)*log(temp(nout))))
                log10psat = log10psat + osimpgroup(i,k)*bk_T
              ENDDO
              pvap=exp(log10Psat*log(10.))
              !Cstar_T(i)=pvap*wmol(i)/(Ratm*293) * 1.e+9
              ! for wall
              Cstar_T(i)=pvap*mw_wall/(Ratm*293) * 1.e+9
! for 298 K
              log10psat = 0.
              DO k=1,mxsim
                bk_T = ((bk(k,1)/temp(nout))+ bk(k,2)+       
     &                  (bk(k,3)*temp(nout))+(bk(k,4)*log(temp(nout))))
                log10psat = log10psat + osimpgroup(i,k)*bk_T
              ENDDO
              pvap=exp(log10Psat*log(10.))
              Cstar_298(i)=pvap*wmol(i)/(Ratm*298) * 1.e+9
            ENDIF

! assign kwg to each species, depending on ICHAM and Cstar (after mtrat.f)
! new parameterization re wall_Krech2016_J
           Cstar = Cstar_T(i)
           KGWA = 0.
            kwg(i) = 0.
            IF(ICHAM.EQ.2)THEN
              KGWA = 1.667E-03 ! s-1
              woucfa = 16.0
              woucfb = 0.6
              Cw(i)  = MAX(10.,(MIN(1e4,woucfa*Cstar**woucfb)))
              kwg(i) = KGWA*Cstar / Cw(i)
            ELSE IF (ICHAM.EQ.3) THEN
              KGWA = 1.000E-03 ! s-1
              IF(Cstar.GT.0.1e3)THEN
                woucfa = 19.
                woucfb = 0.6
              ELSE
                woucfa = 6.49
                woucfb = 0.75
              ENDIF
              Cw(i) = MAX(0.1,(MIN(1e4,woucfa*Cstar*woucfb)))
              kwg(i) = KGWA*Cstar / Cw(i)
            ENDIF
          ENDIF ! oTb(i) > 0

! --- assess PEROx & MEPERO concentration time-series ---

          IF(perotyp(i).GT.0)THEN
            DO k=1,nout
              cro2(k,perotyp(i))=cro2(k,perotyp(i))+cout(k,i)
            ENDDO
          ENDIF
          memo=j
          EXIT
        ENDIF  ! chrsp = name
      ENDDO ! j=memo,ndic
      ENDDO !i=i1,i2 (usually 1,numsp)

!------------------------------------------------------
!--- normalize elemental ratios to total conc ---

      DO k=1,nout
             OtoC_gas(k) = OtoC_gas(k)/ctotgas(k)
             HtoC_gas(k) = HtoC_gas(k)/ctotgas(k)
             NtoC_gas(k) = NtoC_gas(k)/ctotgas(k)
             OtoC_aer(k) = OtoC_aer(k)/corgaer(k)
             HtoC_aer(k) = HtoC_aer(k)/corgaer(k)
             NtoC_aer(k) = NtoC_aer(k)/corgaer(k)
             OtoC_wal(k) = OtoC_wal(k)/corgwal(k)
             HtoC_wal(k) = HtoC_wal(k)/corgwal(k)
             NtoC_wal(k) = NtoC_wal(k)/corgwal(k)
      ENDDO ! k=1,ndat

! --- apply detection limit cutoff to output ---
!      WHERE(cout(:,:).LT.cutoff) cout=0.

!------------------------------------------------------
      WRITE(6,*) '995 : done C and function for each species'

!------------------------------------------------------
! --- read indexed j-value output 
!------------------------------------------------------
      PRINT*,"reading jvals from nc file"
      idchrom = 0
      CALL eznc_get_1Dint(ncid_in,"idchrom",idchrom,1,njchrom)
      PRINT*,idchrom

!------------
! ascii: construct character variable jlabel, for output later
!------------
      DO i=1,njchrom
        IF (idchrom(i).LE.9) THEN
          jlabel(i)(1:4) = '0000'
          WRITE(jlabel(i)(5:5),'(i1)') idchrom(i)
        ELSE IF (idchrom(i).LE.99) THEN
          jlabel(i)(1:3) = '000'
          WRITE(jlabel(i)(4:5),'(i2)') idchrom(i)
        ELSE IF (idchrom(i).LE.999) THEN
          jlabel(i)(1:2) = '00'
          WRITE(jlabel(i)(3:5),'(i3)') idchrom(i)
        ELSE IF (idchrom(i).LE.9999) THEN
          jlabel(i)(1:1) = '0'
          WRITE(jlabel(i)(2:5),'(i4)') idchrom(i)
        ELSE 
          WRITE(jlabel(i)(1:5),'(i5)') idchrom(i)
        ENDIF
      ENDDO

!------------------------------------------------------
! --- find indices for specific j-values from general array
!------------------------------------------------------

      DO i=1,njchrom
        IF(idchrom(i).EQ.  3) ijO3   = i
        IF(idchrom(i).EQ.  4) ijNO2  = i
        IF(idchrom(i).EQ.  6) ijNO3  = i
        IF(idchrom(i).EQ. 11) ijH2O2 = i
        IF(idchrom(i).EQ. 12) ijHONO = i
        IF(idchrom(i).EQ.110) ijCH2O = i
      ENDDO

!------------------------------------------------------
! --- write more ascii header, only on first data read ---
      IF(ihdr.EQ.0)THEN

      PRINT*," Writing Internal Header...."

! --- number of output times: character string (for header formatting)
        IF(nout.LT.10) THEN
          ALLOCATE(character(1) :: cnout)
          WRITE(cnout,'(i1)') nout
        ELSEIF(nout.LT.100) THEN
          ALLOCATE(character(2) :: cnout)
          WRITE(cnout,'(i2)') nout
        ELSEIF(nout.LT.1000) THEN
          ALLOCATE(character(3) :: cnout)
          WRITE(cnout,'(i3)') nout
        ELSE
          PRINT*," # output times > 999 : stopping"
          STOP
        ENDIF
        PRINT*,"nout, cnout = ",nout, cnout
        IF(cnout(1:1).EQ."0")STOP
     
        c1 = ','

! --- header lines that depend on value of cnout --
      WRITE(uou,*)'-----------AUXILIARY DATA COLUMN HEADERS-----------'
      WRITE(uou,*) 
     &            'measure, units, value (x '//cnout//' columns)'
      WRITE(uou,*)'---------------AUXILIARY DATA TYPES-----------------'
      WRITE(uou,*)'[s],[s],[r](x '//cnout//' columns)'

      WRITE(uou,*)'------------OUTPUT DATA COLUMN HEADERS--------------'

      IF(fgcodes)THEN
        IF(radicals)THEN
          WRITE(uou,*)'name, molec_formula, MW, Cstar_298,'// 
     &          'Tb, dB, DVa, kwg, kHenry, dH2Odx, reacf, '//
     &          'F_A, F_D, F_E, F_EE, '//
     &          'F_G, F_H, F_K, F_N, F_O, F_P, F_R, F_T, F_U, F_V, '//
     &          'F_X, rad, RO2, ifg, phz, '// 
     &          'concentration(molec/cc)(x '//cnout//' columns), '//
     &          'SMILES, chemical_formula'
        ELSE !(fgcodes.AND..NOT.(radicals))
          WRITE(uou,*)'name, molec_formula, MW, Cstar_298,'// 
     &          'Tb, dB, DVa, kwg, kHenry, dH2Odx, reacf, '//
     &          'F_A, F_D, F_E, F_EE, '//
     &          'F_G, F_H, F_K, F_N, F_O, F_P, F_R, F_T, F_U, F_V, '//
     &          'F_X, ifg, phz, '// 
     &          'concentration(molec/cc)(x '//cnout//' columns), '//
     &          'SMILES, chemical_formula'
        ENDIF
      ELSE !(.NOT.(fgcodes))THEN
        WRITE(uou,*)'name, molec_formula, MW, Cstar_298,'// 
     &          'Tb, dB, DVa, kwg, kHenry, dH2Odx, reacf, '//
     &          'ifg, phz, '//
     &          'concentration(molec/cc)(x '//cnout//' columns), '//
     &          'SMILES, chemical_formula'
      ENDIF

      WRITE(uou,*)'----------------OUTPUT DATA TYPES-------------------'

      IF(fgcodes)THEN
        IF(radicals)THEN
          WRITE(uou,*)'[s],[s],[r],'//             ! name, molfor, molwt
     &             '[r],[r],[r],[r],'//           ! Cstar, pvap, Dva
     &             '[r],[r],[r],[r],'//            ! kwg, Henry(3)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(1:5)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(6:10)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(11:15)
     &             '[i],[i],'//                    ! rad, RO2
     &             '[i],[i],'//                    ! ifg,phz
     &             '[r](x '//cnout//' columns),'// ! conc(time)
     &             '[s],[s]'                       ! smile, chem formulat
        ELSE !(fgcodes.AND..NOT.(radicals))
          WRITE(uou,*)'[s],[s],[r],'//             ! name, molfor, molwt
     &             '[r],[r],[r],[r],'//           ! Cstar, pvap, Dva
     &             '[r],[r],[r],[r],'//            ! kwg, Henry(3)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(1:5)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(6:10)
     &             '[i],[i],[i],[i],[i],'//        ! fgcodes(11:15)
     &             '[i],[i],'//                    ! ifg,phz
     &             '[r](x '//cnout//' columns),'// ! conc(time)
     &             '[s]'                           ! chem formulat
        ENDIF
      ELSE !(.NOT.(fgcodes))THEN
          WRITE(uou,*)'[s],[s],[r],'//             ! name, molfor, molwt
     &             '[r],[r],[r],[r],'//           ! Cstar, pvap, Dva
     &             '[r],[r],[r],[r],'//            ! kwg, Henry(3)
     &             '[i],[i],'//                    ! rad, RO2
     &             '[i],[i],'//                    ! ifg,phz
     &             '[r](x '//cnout//' columns),'// ! conc(time)
     &             '[s],[s]'                       ! smile, chem formulat
      ENDIF !(fgcodes)

      IF(cutoff.GT.0.)THEN
      WRITE(uou,'(a35,ES10.3)') 'cutoff applied (units = molec/cc): ',
     &            cutoff
      ENDIF

      WRITE(uou,*)'------------------AUXILIARY DATA--------------------'

      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'time       , seconds ,', time(1:nout)
      WRITE(uou,'(a22,1x,'//cnout//'(F10.1",",1x))') 
     &         'temperature , Kelvin ,', temp(1:nout)
      WRITE(uou,'(a22,1x,'//cnout//'(F10.1",",1x))') 
     &         'humidity    , %      ,', rhum(1:nout)

      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'OHexpos, molec cm-3 s,', OHexp(1:nout)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'O3expos, molec cm-3 s,', O3exp(1:nout)
      WRITE(uou,'(a23,1x,'//cnout//'(ES10.3",",1x))') 
     &         'NO3expos, molec cm-3 s,', NO3exp(1:nout)

      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO1, molec cm-3    ,', cro2(1:nout,1)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO2, molec cm-3    ,', cro2(1:nout,2)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO3, molec cm-3    ,', cro2(1:nout,3)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO4, molec cm-3    ,', cro2(1:nout,4)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO5, molec cm-3    ,', cro2(1:nout,5)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO6, molec cm-3    ,', cro2(1:nout,6)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO7, molec cm-3    ,', cro2(1:nout,7)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO8, molec cm-3    ,', cro2(1:nout,8)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'PERO9, molec cm-3    ,', cro2(1:nout,9)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'MEPERO, molec cm-3   ,', cro2(1:nout,10)

      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'gas phz mass, ug m-3 ,', mgas_out(1:nout)

      IF(soa_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'aerosol mass, ug m-3 ,', maer_out(1:nout)
      
      IF(wall_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'wall phz mass, ug m-3,', mwal_out(1:nout)

      IF(soa_fg.NE.0)
     & WRITE(uou,'(a28,1x,'//cnout//'(ES10.3",",1x))') 
     &         '[oa+seed](t-1), molec cm-3, ',ctotaer(1:nout)
      
      IF(wall_fg.NE.0) 
     & WRITE(uou,'(a24,1x,'//cnout//'(ES10.3",",1x))') 
     &         '[org_wall], molec cm-3, ',corgwal(1:nout)

      IF(soa_fg.NE.0) THEN
      WRITE(uou,'(a23,1x,'//cnout//'(ES10.3",",1x))') 
     &         'particle radius Rp, cm,',Rp(1:nout)
      WRITE(uou,'(a30,1x,'//cnout//'(ES10.3",",1x))') 
     &         'mean oa+seed molwt, g.mole-1, ',mwaer(1:nout)
      ENDIF

      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'O_to_C_gas, ratio    ,', OtoC_gas(1:nout)
      IF(soa_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'O_to_C_aer, ratio    ,', OtoC_aer(1:nout)
      IF(wall_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'O_to_C_wall, ratio   ,', OtoC_wal(1:nout)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'H_to_C_gas, ratio    ,', HtoC_gas(1:nout)
      IF(soa_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'H_to_C_aer, ratio    ,', HtoC_aer(1:nout)
      IF(wall_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'H_to_C_wall, ratio   ,', HtoC_wal(1:nout)
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'N_to_C_gas, ratio    ,', NtoC_gas(1:nout)
      IF(soa_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'N_to_C_aer, ratio    ,', NtoC_aer(1:nout)
      IF(wall_fg.NE.0) 
     & WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &         'N_to_C_wall, ratio   ,', NtoC_wal(1:nout)

      DO i=1,njchrom 
      WRITE(uou,'(a22,1x,'//cnout//'(ES10.3",",1x))') 
     &        'j'//jlabel(i)//' , s-1         ,', ojchrom(1:nout,i)
      ENDDO

* -----------------------------
! --- NetCDF WRITE INITIALIZATION DATA --
* -----------------------------

      CALL eznc_put_0Dint(ncid_out,"autoox_fg",autoox_fg)
      CALL eznc_put_0Dint(ncid_out,"depos_fg",depos_fg)
      CALL eznc_put_0Dint(ncid_out,"dimer_fg",dimer_fg)
      CALL eznc_put_0Dint(ncid_out,"dyn_fg",dyn_fg)
      CALL eznc_put_0Dint(ncid_out,"flgdhf",flgdhf)
      CALL eznc_put_0Dint(ncid_out,"high_NOxfg",high_NOxfg)
      CALL eznc_put_0Dint(ncid_out,"zero_NOxfg",zero_NOxfg)
      CALL eznc_put_0Dint(ncid_out,"isomer_fg",isomer_fg)
      CALL eznc_put_0Dint(ncid_out,"OFR_fg",OFR_fg)
      CALL eznc_put_0Dint(ncid_out,"pvap_fg",pvap_fg)
      CALL eznc_put_0Dint(ncid_out,"ro2_fg",ro2_fg)
      CALL eznc_put_0Dint(ncid_out,"seedtyp_fg",seedtyp_fg)
      CALL eznc_put_0Dint(ncid_out,"soa_fg",soa_fg)
      CALL eznc_put_0Dint(ncid_out,"wall_fg",wall_fg)

      CALL eznc_put_0Dreal(ncid_out,"TLEN",TLEN)
      CALL eznc_put_0Dreal(ncid_out,"NDEN",NDEN)
      IF (OFR_fg.GT.0)THEN
        CALL eznc_put_0Dreal(ncid_out,"f185",f185)
        CALL eznc_put_0Dreal(ncid_out,"f254",f254)
        CALL eznc_put_0Dreal(ncid_out,"WATR",WATR)
      ENDIF
      IF (soa_fg.GT.0)THEN
        CALL eznc_put_0Dreal(ncid_out,"GAMM",GAMM)
        CALL eznc_put_0Dreal(ncid_out,"SEED",SEED)
        CALL eznc_put_0Dreal(ncid_out,"NVMW",NVMW)
        CALL eznc_put_0Dreal(ncid_out,"NVRO",NVRO)
      ENDIF
      IF (wall_fg.GT.0)THEN
        CALL eznc_put_0Dint(ncid_out,"ICHAM",ICHAM)
        CALL eznc_put_0Dreal(ncid_out,"KGWA",KGWA)
      ENDIF

!---------------------------------------------------
! --- NetCDF WRITE AUX DATA --
!---------------------------------------------------

      CALL eznc_put_1dreal(ncid_out,"time",time,1,nout)
      CALL eznc_put_1dreal(ncid_out,"temperature",temp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"humidity",rhum,1,nout)
      CALL eznc_put_1Dint (ncid_out,"idchrom",idchrom,1,njchrom)
      CALL eznc_put_2Dreal(ncid_out,"jchrom",ojchrom(1:nout,1:njchrom),
     &                                               1,nout,1,njchrom)
!      CALL eznc_put_1Dreal(ncid_out,"jO3",ojchrom(1:nout,ijO3),
!     &                                            1,nout)
!      CALL eznc_put_1Dreal(ncid_out,"jNO2",ojchrom(1:nout,ijNO2),
!     &                                             1,nout)
!      CALL eznc_put_1Dreal(ncid_out,"jNO3",ojchrom(1:nout,ijNO3),
!     &                                             1,nout)
!      CALL eznc_put_1Dreal(ncid_out,"jH2O2",ojchrom(1:nout,ijH2O2),
!     &                                              1,nout)
!      CALL eznc_put_1Dreal(ncid_out,"jHONO",ojchrom(1:nout,ijHONO),
!     &                                              1,nout)
!      CALL eznc_put_1Dreal(ncid_out,"jCH2O",ojchrom(1:nout,ijCH2O),
!     &                                              1,nout)

      CALL eznc_put_1dreal(ncid_out,"OHexpos",OHexp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"O3expos",O3exp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"NO3expos",NO3exp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"O3expos",O3exp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"NO3expos",NO3exp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO1",cro2(1:nout,1),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO2",cro2(1:nout,2),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO3",cro2(1:nout,3),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO4",cro2(1:nout,4),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO5",cro2(1:nout,5),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO6",cro2(1:nout,6),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO7",cro2(1:nout,7),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO8",cro2(1:nout,8),1,nout)
      CALL eznc_put_1dreal(ncid_out,"PERO9",cro2(1:nout,9),1,nout)
      CALL eznc_put_1dreal(ncid_out,"MEPERO",cro2(1:nout,10),1,nout)
      CALL eznc_put_1dreal(ncid_out,"gas_phz_mass",mgas_out,1,nout)
      CALL eznc_put_1dreal(ncid_out,"aerosol_mass",maer_out,1,nout)
      CALL eznc_put_1dreal(ncid_out,"wall_phz_mass",mwal_out,1,nout)
      CALL eznc_put_1dreal(ncid_out,"oa+seed_conc",ctotaer,1,nout)
      CALL eznc_put_1dreal(ncid_out,"org_wall_conc",corgwal,1,nout)
      CALL eznc_put_1dreal(ncid_out,"Rp",Rp,1,nout)
      CALL eznc_put_1dreal(ncid_out,"oa+seed_molwt",mwaer,1,nout)
      CALL eznc_put_1dreal(ncid_out,"O_to_C_gas",OtoC_gas,1,nout)
      CALL eznc_put_1dreal(ncid_out,"H_to_C_gas",HtoC_gas,1,nout)
      CALL eznc_put_1dreal(ncid_out,"N_to_C_gas",NtoC_gas,1,nout)
      CALL eznc_put_1dreal(ncid_out,"O_to_C_aer",OtoC_aer,1,nout)
      CALL eznc_put_1dreal(ncid_out,"H_to_C_aer",HtoC_aer,1,nout)
      CALL eznc_put_1dreal(ncid_out,"N_to_C_aer",NtoC_aer,1,nout)
      CALL eznc_put_1dreal(ncid_out,"O_to_C_wall",OtoC_wal,1,nout)
      CALL eznc_put_1dreal(ncid_out,"H_to_C_wall",HtoC_wal,1,nout)
      CALL eznc_put_1dreal(ncid_out,"N_to_C_wall",NtoC_wal,1,nout)

      DEALLOCATE ( jlabel, idchrom, jchrom, ojchrom )

! DEBUG
!      CALL close_ncfile(ncid_out)
!      PRINT*,"DEBUG 3"
!      STOP

!===========================================================================
! --- write ascii output by species (every data read) ---
!===========================================================================

      WRITE(uou,*)'-------------------OUTPUT DATA----------------------'
        ihdr = 1
      ENDIF ! (ihdr.EQ.0)

! --- loop species ---

      PRINT*," 2nd loop : working....."

      DO i=i1,i2

        PRINT*,i," 2nd"

! TREAT RADICALS IN GAS PHASE ONLY
        IF(INDEX(ocode(i),'.').NE.0.AND.iphz(i).GT.1)CYCLE

        molform=""
        ifg = 0
        radtyp = 0
        fungrp(:) = 0

        IF(ochem(i)(1:3).EQ."#mm".OR.ochem(i)(1:3).EQ."#bb")THEN 
          tchem(1:lfo)=ochem(i)(4:lfo)//'   '
          ochem(i)=tchem
        ENDIF
       
!====FIND SMILES STRING====
! Special species: specify molform and don't call smiles
        IF( (ochem(i)(1:1).EQ."#".AND.
! allow explicit formulae : aromatics
     &       ochem(i)(2:3).NE."mm".AND.
! allow explicit formulae : non-aromatics
     &       ochem(i)(2:2).NE."C")  .OR.
! special cases: CH2OOx
     &      (ochem(i)(1:6).EQ."#CH2OO".AND.
     &       (ochem(i)(7:7).NE." ".AND.ochem(i)(8:8).EQ." ")).OR.
     &     (INDEX(ochem(i),"ISO").NE.0).OR.
     &     (INDEX(ochem(i),"MOD").NE.0).OR.
     &     (INDEX(ochem(i),"X").NE.0).OR.
     &     (INDEX(ochem(i),"Y").NE.0)    )THEN
          PRINT*,i,"is a special species: ",ochem(i)
          SELECT CASE(ochem(i)(1:6))
            CASE("#PCATE")
              molform="C8H16O5    "
              wmol(i) = 192.21
            CASE("#MCATE")
              molform="C7H12O5    "
              wmol(i) = 176.17
            CASE("#CATEC")
              molform="C6H10O5    "
              wmol(i) = 162.14
            CASE("#TLFUO")
              molform="C5H6O5     "
              wmol(i) =146.10
            CASE("#BZFUO")
              molform="C4H4O5     "
              wmol(i) = 132.07
            CASE("#GLYOO")
              molform="C2H2O3     "
              wmol(i) = 74.04
            CASE("#lostc")
              molform="Cx         "
              wmol(i) = 0.00
            CASE DEFAULT
              molform="unknown    "
              wmol(i) = 0.00
          END SELECT

          SELECT CASE(oname(i)(1:6))
!   #CH2OO
            CASE("AR0495")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!   #CH2OOA
            CASE("C50326")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!   #CH2OOB
            CASE("AR0489")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!   #CH2OOC
            CASE("C50040")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!   #CH2OOE
            CASE("C50004")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!   #CH2OOG  
            CASE("C50020")  
              molform="CH2O2     "
                wmol(i) =  46.0100
!#CISOPA
            CASE("C50008")  
              molform="C5H9O     "
                wmol(i) =  85.1244
!#TISOPA 
            CASE("C50013") 
              molform="C5H9O     "
                wmol(i) = 85.1244
!#CISOPC
            CASE("C50009")  
              molform="C5H9O     "
                wmol(i) =  85.1244
!#TISOPC
            CASE("C50014") 
              molform="C5H9O     "
                wmol(i) = 85.1244
!#ISOPAO
            CASE("C50108") 
              molform="C5H9O2    "
                wmol(i) = 101.1238
!#CISOPAO
            CASE("C50072")  
              molform="C5H9O2    "
                wmol(i) = 101.1238
!#CISOPCO
            CASE("C50080")  
              molform="C5H9O2    "
                wmol(i) = 101.1238
!#ISOPAO2
            CASE("C50048") 
              molform="C5H9O3    "
                wmol(i) = 117.1232
!#CISOPAO2
            CASE("C50031")  
              molform="C5H9O3    "
                wmol(i) = 117.1232
!#ISOPCO2 
            CASE("C50049") 
              molform="C5H9O3    "
                wmol(i) = 117.1232
!#CISOPCO2
            CASE("C50033")  
              molform="C5H9O3    "
                wmol(i) = 117.1232
          END SELECT

        ELSE
! remove '#' from formula
          IF(ochem(i)(1:1).EQ."#")THEN 
             ochem(i)(1:LEN_TRIM(ochem(i))-1) = 
     &       ochem(i)(2:LEN_TRIM(ochem(i)))
             ochem(i)(LEN_TRIM(ochem(i)):LEN_TRIM(ochem(i)))=" "
          ENDIF

! --- find SMILES string ---
          smile(1:lfo) = " "
          CALL smiles(ochem(i),smile)
          spos = INDEX(smile,' ')-1

          iend = 11
          ipos = 1

          IF(ncarbon(i).GT.9)THEN
            WRITE(x,'(i2)') ncarbon(i) 
            WRITE(molform(1:3),'(a1,a2)')'C',x
            ipos = ipos+3
          ELSE IF (ncarbon(i).GT.0)THEN
            WRITE(x,'(i1)') ncarbon(i) 
            WRITE(molform(1:2),'(a1,a1)')'C',x
            ipos = ipos+2
          ELSE
            ifg = 1
!! rename "HO" to "OH"
!            IF(oname(i)(1:3).EQ."HO ")oname(i)(1:3)="OH "
          ENDIF!(ncarbon(i).GT.9)THEN

          IF(nhydrogen(i).GT.9)THEN
            WRITE(y,'(i2)') nhydrogen(i) 
            WRITE(molform(ipos:ipos+2),'(a1,a2)')'H',y
            ipos = ipos+3
          ELSE IF (nhydrogen(i).GT.0)THEN
            WRITE(y,'(i1)') nhydrogen(i) 
            WRITE(molform(ipos:ipos+1),'(a1,a1)')'H',y
            ipos = ipos+2
          ELSE IF (ncarbon(i).EQ.1) THEN
            ifg = 1
          ENDIF!(nhydrogen(i).GT.9)THEN

! TREAT INORGANICS IN GAS PHASE ONLY
          IF(ifg.EQ.1.AND.iphz(i).GT.1)CYCLE

          IF(nnitrogen(i).GT.0)THEN
            WRITE(w,'(i1)') nnitrogen(i) 
            WRITE(molform(ipos:ipos+1),'(a1,a1)') 'N',w
            ipos = ipos+2
          ENDIF !(nnitrogen(i).GT.0)THEN

          IF(noxygen(i).GT.9)THEN
            WRITE(z,'(i2)') noxygen(i) 
            WRITE(molform(ipos:ipos+2),'(a1,a2)') 'O',z
            ipos = ipos+3
          ELSE IF(noxygen(i).GT.0)THEN
            WRITE(z,'(i1)') noxygen(i) 
            WRITE(molform(ipos:ipos+1),'(a1,a1)') 'O',z
            ipos = ipos+2
          ENDIF !(noxygen(i).GT.0)THEN

          IF(nsulfur(i).GT.0)THEN
            WRITE(v,'(i1)') nsulfur(i) 
            WRITE(molform(ipos:ipos+1),'(a1,a1)') 'S',v
            ipos = ipos+2
          ENDIF !(nsulfur(i).GT.0)THEN

          molform(ipos:iend)=' '

! output is good
          !WRITE(6,'(a11,1x,a15,1x,a120)') molform, ocode(i),ochem(i)

        ENDIF !(named species)

! --- find functions & radicals ---

        DO ipos=1,lfl

          SELECT CASE (ocode(i)(ipos:ipos))
          CASE ('A')  ! ACID
            fungrp(1)=fungrp(1)+1
          CASE ('D')  ! ALDEHYDE
            fungrp(2)=fungrp(2)+1
          CASE ('E')  ! ETHER: 
            IF(ocode(i)(ipos-1:ipos-1).EQ.'E') THEN ! already on 2nd 'E'
             CONTINUE
            ENDIF
            IF(ocode(i)(ipos-1:ipos-1).NE.'E'.AND.
     &         ocode(i)(ipos+1:ipos+1).NE.'E') THEN  ! 3 = simple ether
              fungrp(3)=fungrp(3)+1
            ENDIF
            IF(ocode(i)(ipos+1:ipos+1).EQ.'E') THEN ! 14 = -O-O- bridge
              fungrp(4)=fungrp(4)+1
            ENDIF
          CASE ('G')  ! PERACID
            fungrp(5)=fungrp(5)+1
          CASE ('H')  ! HYDROPEROXY
            fungrp(6)=fungrp(6)+1
          CASE ('K')  ! KETONE
            fungrp(7)=fungrp(7)+1
          CASE ('N')  ! NITRATE
            fungrp(8)=fungrp(8)+1
          CASE ('O')  ! ALCOHOL
            fungrp(9)=fungrp(9)+1
          CASE ('P')  ! PAN
            fungrp(10)=fungrp(10)+1
!AROMATIC species "c" : designate code "R", below
            fungrp(11)=fungrp(11)
          CASE ('T')  ! RING (NEW)
            fungrp(12)=fungrp(12)+1
          CASE ('U')  ! DOUBLE BOND
            fungrp(13)=fungrp(13)+1
          CASE ('V')  ! NITRITE
            fungrp(14)=fungrp(14)+1
          CASE ('X')  ! =CdO
            fungrp(15)=fungrp(15)+1
          CASE ('1')  ! oxy radical
            radtyp= 1
          CASE ('2')  ! peroxy radical
            radtyp= 2
          CASE ('3')  ! acyl peroxy radical
            radtyp= 3
          CASE ('4')  ! Criegge radical
            radtyp= 4
          CASE (' ')  ! end of code string
            EXIT
          END SELECT
        ENDDO !ipos=1,lfl

!AROMATIC species "c" : designate code "R" as fungrp 11
        IF(INDEX(ochem(i),"c").NE.0) fungrp(11)=fungrp(11)+1

!Aliphatic rings if not already found
        IF(fungrp(12).EQ.0) THEN
          IF(INDEX(ochem(i),"C1").NE.0) fungrp(12)=1
          IF(INDEX(ochem(i),"C2").NE.0) fungrp(12)=2
        ENDIF

! --- length of output formula: character string ---
        ipos = INDEX(ochem(i),' ')-1
        WRITE(cipos,'(i3)') ipos

        spos = INDEX(smile,' ')-1
        WRITE(cspos,'(i3)') spos
        !WRITE(6,'(i3,1x,a'//cipos//')')i,ochem(i)(1:ipos)

! --- kphase depends on phase
!        SELECT CASE (iphz(i))
!          CASE (1) 
!            okphase(i) = okHenry(i) 
!          CASE (2) 
!            okphase(i) = 0.
!          CASE (3) 
!            okphase(i) = kwg(i)
!          CASE DEFAULT
!            okphase(i) = 0.
!        END SELECT

! --- write all data ---
! name, molfor, molwt
! pvap (1:2), Henry(1:3)
! fgcodes(1:15)
! rad, RO2, phz
! conc(time)
! chem formula

!        IF(INDEX(molform,"C2H4O6").EQ.1)THEN
!---------------------------------
! OUTPUT COMMON MOLECULAR INFO 
!---------------------------------

      IF ((INDEX(oname(i),"X").NE.1).AND.
     &    (INDEX(oname(i),"Y").NE.1).AND.
     &    (INDEX(oname(i),"MOD").EQ.0).AND.
     &    (INDEX(oname(i),"XCLOST").EQ.0)) THEN
! (don't put these values into NetCDF arrays, only into ascii file)

      CALL eznc_put_0Dchar_into1D(ncid_out,"name",oname(i),lco,i)
      CALL eznc_put_0Dchar_into1D(ncid_out,"molec_formula",
     &                                      molform,lmf,i)
      CALL eznc_put_0Dchar_into1D(ncid_out,"SMILES",
     &                                      smile,spos,i)
      CALL eznc_put_0Dchar_into1D(ncid_out,"chemical_formula",
     &                                      ochem(i),
     &                             LEN_TRIM(ochem(i)),i)

      CALL eznc_put_0Dint_into1D(ncid_out,"rad",radtyp,i)
      CALL eznc_put_0Dint_into1D(ncid_out,"RO2",perotyp(i),i)
      CALL eznc_put_0Dint_into1D(ncid_out,"ifg",ifg,i)
      CALL eznc_put_0Dint_into1D(ncid_out,"phz",iphz(i),i)
      CALL eznc_put_1Dint_into2D(ncid_out,"fungrp",fungrp(:),1,nfgrp,i)

      CALL eznc_put_0Dreal_into1D(ncid_out,"MW",wmol(i),i)

      IF(radtyp.EQ.0)THEN
      CALL eznc_put_0Dreal_into1D(ncid_out,"Cstar_298",Cstar_298(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"Tb",oTb(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"dB",odB(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"DVa",DVa(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"kwg",kwg(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"kHenry",okhenry(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"dH2Odx",odh2odx(i),i)
      CALL eznc_put_0Dreal_into1D(ncid_out,"reacf",oreacf(i),i)
      ENDIF

      CALL eznc_put_1Dreal_into2D(ncid_out,"concentration",
     &                                  cout(1:nout,i),1,nout,i)
      ENDIF
!----

        IF(fgcodes)THEN

          IF(radicals)THEN

            WRITE(uou,'(a6,a1,1x,a11,a1,1x,
     &            F8.4,a1,1x, ES10.3,a1,1x, F5.1,a1,1x, F6.4,a1,1x,
     &            F7.2,a1,1x,ES8.2,a1,1x,
     &            3(ES8.2,a1,1x),
     &            15(i1,a1,1x),
     &            4(i1,a1,1x),
     &            '//cnout//'(ES12.5",",1x),
     &            a'//cspos//',a1,1x, a'//cipos//')') 
     &          oname(i),',',molform,',',
     &          wmol(i),',',Cstar_298(i),',',oTb(i),',', odB(i),',', 
     &          DVa(i),',',kwg(i),',',
     &          okHenry(i),',',odh2odx(i),',',oreacf(i),',', 
     &          fungrp(1),',',fungrp(2),',',fungrp(3),',',
     &          fungrp(4),',',fungrp(5),',',fungrp(6),',',
     &          fungrp(7),',',fungrp(8),',',fungrp(9),',',
     &          fungrp(10),',',fungrp(11),',',fungrp(12),',',
     &          fungrp(13),',',fungrp(14),',',fungrp(15),',', 
     &          radtyp,',',perotyp(i),',',
     &          ifg,',',iphz(i),',',
     &          cout(1:nout,i),
     $          smile(1:spos),',',
     &          ochem(i)(1:ipos)


          ELSE !(.NOT.radicals)
            IF(INDEX(ochem(i),'.').EQ.0)THEN  ! not a radical
              WRITE(uou,'(a6,a1,1x,a11,a1,1x,
     &            F8.4,a1,1x, ES10.3,a1,1x, F5.1,a1,1x, F6.4,a1,1x,
     &            F7.2,a1,1x,ES8.2,a1,1x,
     &              3(ES8.2,a1,1x),
     &              15(i1,a1,1x),
     &              2(i1,a1,1x),
     &              '//cnout//'(ES12.5",",1x),
     &              a'//cspos//',a1,1x, a'//cipos//')') 
     &          oname(i),',',molform,',',
     &          wmol(i),',',Cstar_298(i),',',oTb(i),',', odB(i),',', 
     &          DVa(i),',',kwg(i),',',
     &          okHenry(i),',',odh2odx(i),',',oreacf(i),',', 
     &          fungrp(1),',',fungrp(2),',',fungrp(3),',',
     &          fungrp(4),',',fungrp(5),',',fungrp(6),',',
     &          fungrp(7),',',fungrp(8),',',fungrp(9),',',
     &          fungrp(10),',',fungrp(11),',',fungrp(12),',',
     &          fungrp(13),',',fungrp(14),',',fungrp(15),',', 
     &          ifg,',',iphz(i),',',
     &          cout(1:nout,i),
     $          smile(1:spos),',',
     &          ochem(i)(1:ipos)
             ENDIF ! not a radical
          ENDIF ! (radicals)

        ELSE !(.NOT.(fgcodes))THEN
          WRITE(uou,'(a6,a1,1x,a11,a1,1x,
     &            F8.4,a1,1x, ES10.3,a1,1x, F5.1,a1,1x, F6.4,a1,1x,
     &            F7.2,a1,1x,ES8.2,a1,1x,
     &              3(ES8.2,a1,1x),
     &              2(i1,a1,1x),
     &              '//cnout//'(ES12.5",",1x),
     &              a'//cspos//',a1,1x, a'//cipos//')') 
     &              oname(i),',',molform,',',
     &              wmol(i),',',Cstar_298(i),',',oTb(i),',', odB(i),',', 
     &              DVa(i),',',kwg(i),',',
     &              okHenry(i),',',odh2odx(i),',',oreacf(i),',', 
     &              ifg,',',iphz(i),',',
     &              cout(1:nout,i),
     $              smile(1:spos),',',
     &              ochem(i)(1:ipos)
        ENDIF ! (fgcodes)
!        ENDIF!(INDEX(molform,"(specified)").EQ.1)THEN

! spare code for reinstating 2 Dep params
!     &            2(F6.2,a1,1x),
!     &          Tb(i),',', dB(i),',',
!     &           okHenry(i),',',odh2odx(i),',',oreacf(i)

      ENDDO !i= i1,i2 (1,numsp)

      IF(asciiopflag)THEN
        WRITE(uou,*)'END'
        CLOSE(uou)
      ENDIF

      CALL close_ncfile(ncid_out)

      DEALLOCATE ( cnout )
      DEALLOCATE ( perotyp, iphz )
      DEALLOCATE ( wmol, Cstar_298, Cstar_T, DVa, Cw, Tb, dB, oTb, oDb,
     &             kHenry, okHenry, kwg, kga, okphase,
     &             simpgroup, bk,
     &             dh2odx, odh2odx, reacf, oreacf )
      DEALLOCATE ( ncarbon, nnitrogen, noxygen, nhydrogen, nsulfur )
      DEALLOCATE ( name, oname, chem, ochem, code, ocode, chrsp, 
     &             satid, chrsp_H, chrsp_ro2, idchemro2 )
      DEALLOCATE ( cgas3, cgas, cout, caer, psat, cro2 )

*  ====================================================================
      END PROGRAM ncplot
*  ===================================================================
!==============================================================!
! PURPOSE: COLLECTION OF ROUTINES TO HANDLE NETCDF-            !
!          FORMAT INPUT & OUTPUT                               !
!          FOR GECKO-A BOX MODEL                               !
! Author: Julia Lee-Taylor, NCAR, January 2018                 !
! Updates:                                                     !
! 2019-April-11 : removed unused variables                     !
!==============================================================!
! ASSUMPTION: variable dimensions, size, type are known        !
!             If this is not the case, the info can be         ! 
!             found using 2 successive calls, e.g.             !
!      status = nf90_inq_varid(ncid, "errnam", VarId)          ! 
!      status = nf90_inquire_variable(ncid,VarId,              !  
!     $         xtype,numDims,dimids,numAtts)                  ! 
!                                                              !
! SUBROUTINE LIST:                                             !
! Subroutine names generally begin with "eznc_" (Easy NetCDF)  !
! to distinguish them from NetCDF system calls                 !
! Error handling:
!      SUBROUTINE eznc_handle_err(status,text)
!
! File handling (open/close):
!      SUBROUTINE open_ncfile_new(filename,ncid)
!      SUBROUTINE open_ncfile_readonly(filename,ncid)
!      SUBROUTINE open_ncfile_readwrite(filename,ncid)
!      SUBROUTINE close_ncfile(ncid)
!
! File handling ("define" variable space then "data" for write/read):
!      SUBROUTINE switch_ncfile_to_define_mode(ncid)
!      SUBROUTINE switch_ncfile_to_data_mode(ncid)
!
! Attribute handling ("global" for file metadata)
!      SUBROUTINE eznc_def_sysglobatt(ncid,AttName,Command)
!      SUBROUTINE eznc_def_globalatt(ncid,AttName,AttTxt)
!      SUBROUTINE eznc_get_globalatt(ncid,AttName,AttTxt)
!      SUBROUTINE eznc_del_globalatt(ncid,AttName)
!      SUBROUTINE eznc_copy_globalatt(ncid_in,AttName,ncid)
!      SUBROUTINE eznc_rename_globalatt(ncid,OldName,NewName)
!
! Attribute handling ("local" for variable metadata)
!      SUBROUTINE eznc_def_localatt(ncid,VarName,AttName,AttTxt)
!      SUBROUTINE eznc_get_localatt(ncid,VarName,AttName,AttTxt)
!
! Dimension handling:
!      SUBROUTINE eznc_def_dim(ncid,DimName,dimval)
!      SUBROUTINE eznc_get_dimid(ncid,DimName,dimid)
!      SUBROUTINE eznc_get_dimension(ncid,DimName,dimval)
!
! Variable handling - scalars (define,put,get = declare,write,read):
!      SUBROUTINE eznc_def_0Dint(ncid,VarName)
!      SUBROUTINE eznc_put_0Dint(ncid,VarName,value)
!      SUBROUTINE eznc_get_0Dint(ncid,VarName,value)
!      SUBROUTINE eznc_def_0Dreal(ncid,VarName)
!      SUBROUTINE eznc_put_0Dreal(ncid,VarName,value)
!      SUBROUTINE eznc_get_0Dreal(ncid,VarName,value)
!
! Variable handling - arrays (define,put,get = declare,write,read):
!      SUBROUTINE eznc_def_1Dint(ncid,VarName,DimName)
!      SUBROUTINE eznc_put_1Dint(ncid,VarName,invals,start1,end1)
!      SUBROUTINE eznc_get_1Dint(ncid,VarName,outvals,start1,end1)
!      SUBROUTINE eznc_def_1Dreal(ncid,VarName,DimName)
!      SUBROUTINE eznc_put_1Dreal(ncid,VarName,invals,start1,end1)
!      SUBROUTINE eznc_get_1Dreal(ncid,VarName,outvals,start1,end1)
!      SUBROUTINE eznc_def_1Dchar(ncid,VarName,nchar,DimName)
!      SUBROUTINE eznc_put_1Dchar(ncid,VarName,values,
!     $                                        nchar,start1,end1)
!      SUBROUTINE eznc_get_1Dchar(ncid,VarName,values,
!     $                                        nchar,start1,end1)
!      SUBROUTINE eznc_def_2Dint(ncid,VarName,DimName1,DimName2)
!      SUBROUTINE eznc_put_2Dint(ncid,VarName,invals,
!     $                          start1,end1,start2,end2)
!      SUBROUTINE eznc_get_2Dint(ncid,VarName,outvals,
!     $                          start1,end1,start2,end2)
!      SUBROUTINE eznc_def_2Dreal(ncid,VarName,DimName1,DimName2)
!      SUBROUTINE eznc_put_2Dreal(ncid,VarName,invals,
!     $                           start1,end1,start2,end2)
!      SUBROUTINE eznc_get_2Dreal(ncid,VarName,outvals,
!     $                           start1,end1,start2,end2)
!      SUBROUTINE eznc_def_2Ddbl(ncid,VarName,DimName1,DimName2)
!      SUBROUTINE eznc_def_2Dchar(ncid,VarName,DimName1,DimName2,DimName3)
!      SUBROUTINE eznc_put_2Dchar(ncid,VarName,values,
!     $                                nchar,start1,end1,start2,end2)
!      SUBROUTINE eznc_get_2Dchar(ncid,VarName,nchar,
!     $                                values,start1,end1,start2,end2)
!      SUBROUTINE eznc_def_3Dint(ncid,VarName,DimName1,DimName2,DimName3)
!      SUBROUTINE eznc_put_3Dint(ncid,VarName,invals,
!     $                          start1,end1,start2,end2,start3,end3)
!      SUBROUTINE eznc_get_3Dint(ncid,VarName,outvals,
!     $                         start1,end1,start2,end2,start3,end3)
!      SUBROUTINE eznc_def_3Dreal(ncid,VarName,DimName1,DimName2,DimName3)
!      SUBROUTINE eznc_put_3Dreal(ncid,VarName,invals,
!     $                           start1,end1,start2,end2,start3,end3)
!      SUBROUTINE eznc_get_3Dreal(ncid,VarName,outvals,
!     $                           start1,end1,start2,end2,start3,end3)
!
! NetCDF Fortan manual may be found at:                        !
! https://www.unidata.ucar.edu/software/netcdf/netcdf-4/newdocs/netcdf-f90/index.html#Top
!-------------------------------------------------------!

      SUBROUTINE eznc_handle_err(status,text)

! Return a NetCDF error code for failed NetCDF operation,
!   and terminate program.
! A dictionary of codes is available at:
! https://www.unidata.ucar.edu/software/netcdf/netcdf-4/newdocs/netcdf-c/NetCDF_002d3-Error-Codes.html#NetCDF_002d3-Error-Codes

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER status
      CHARACTER(*) text

! INTERNAL VARIABLES
      INTEGER,PARAMETER :: lout = 11
      CHARACTER(16) errnam
      CHARACTER(60) report
      LOGICAL :: fatal

      status = -(status)
      errnam = "              "    
      report = "                "

      SELECT CASE (status)
        CASE (0)
          errnam = "NC_NOERR        "    
          report = "No Error      "
          fatal = .FALSE.
        CASE (2)
          errnam = "xx_BADFILENAME  "    
          report = "No file found with that name      "
          fatal = .TRUE.
        CASE (33)
          errnam = "NC_EBADID       "
          report = "Not a netcdf id"
          fatal = .TRUE.
        CASE (34)
          errnam = "NC_ENFILE       "
          report = "Too many netcdfs open"
          fatal = .TRUE.
        CASE (35)
          errnam = "NC_EEXIST       "
          report = "netcdf file already exists (NC_NOCLOBBER)"
          fatal = .TRUE.
        CASE (36)
          errnam = "NC_EINVAL       "
          report = "Invalid Argument"
          fatal = .TRUE.
        CASE (37)
          errnam = "NC_EPERM        "
          report = "Write to read only"
          fatal = .TRUE.
        CASE (38)
          errnam = "NC_ENOTINDEFINE "
          report = "Operation not allowed in data mode"
          fatal = .TRUE.
        CASE (39)
          errnam = "NC_EINDEFINE    "
          report = "Operation not allowed in define mode"
          fatal = .TRUE.
        CASE (40)
          errnam = "NC_EINVALCOORDS "
          report = "Index exceeds dimension bound"
          fatal = .TRUE.
        CASE (41)
          errnam = "NC_EMAXDIMS     "
          report = "NC_MAX_DIMS exceeded"
          fatal = .TRUE.
        CASE (42)
          errnam = "NC_ENAMEINUSE   "
          report = "String match to errnam in use"
          fatal = .TRUE.
        CASE (43)
          errnam = "NC_ENOTATT      "
          report = "Attribute not found"
          fatal = .TRUE.
        CASE (44)
          errnam = "NC_EMAXATTS     "
          report = "Not a netcdf data type"
          fatal = .TRUE.
        CASE (46)           
          errnam = "NC_EBADDIM      "
          report = "Invalid dimension id or errnam"
          fatal = .TRUE.
        CASE (47)           
          errnam = "NC_EUNLIMPOS    "
          report = "NC_UNLIMITED in the wrong index"
          fatal = .TRUE.
        CASE (48)           
          errnam = "NC_EMAXVARS     "
          report = "NC_MAX_VARS exceeded"
          fatal = .TRUE.
        CASE (49)           
          errnam = "NC_ENOTVAR      "
          report = "Variable not found"
          fatal = .FALSE.
        CASE (50)           
          errnam = "NC_EGLOBAL      "
          report = "Action prohibited on NC_GLOBAL varid"
          fatal = .TRUE.
        CASE (51)           
          errnam = "NC_ENOTNC       "
          report = "Not a netcdf file"
          fatal = .TRUE.
        CASE (52)           
          errnam = "NC_ESTS         "
          report = "In Fortran, string too short"
          fatal = .TRUE.
        CASE (53)           
          errnam = "NC_EMAXNAME     "
          report = "NC_MAX_NAME exceeded"
          fatal = .TRUE.
        CASE (54)           
          errnam = "NC_EUNLIMIT     "
          report = "NC_UNLIMITED size already in use"
          fatal = .TRUE.
        CASE (55)           
          errnam = "NC_ENORECVARS   "
          report = "nc_rec op when there are no record vars"
          fatal = .TRUE.
        CASE (56)           
          errnam = "NC_ECHAR        "
          report = "Attempt to convert between text & numbers"
          fatal = .TRUE.
        CASE (57)           
          errnam = "NC_EEDGE        "
          report = "Edge+start exceeds dimension bound"
          fatal = .TRUE.
        CASE (58)           
          errnam = "NC_ESTRIDE      "
          report = "Illegal stride"
          fatal = .TRUE.
        CASE (59)           
          errnam = "NC_EBADNAME     "
          report ="Attribute or variable errnam contains illegal chars"
          fatal = .TRUE.
        CASE (60)           
          errnam = "NC_ERANGE       "
          report = "Math result not representable"
          fatal = .FALSE.
        CASE (61)           
          errnam = "NC_ENOMEM       "
          report = "Memory allocation (malloc) failure"
          fatal = .TRUE.
        CASE (62)           
          errnam = "NC_EVARSIZE     "
          report="One or more variable sizes violate format constraints"
          fatal = .TRUE.
        CASE (63)           
          errnam = "NC_EDIMSIZE     "
          report = "Invalid dimension size"
          fatal = .TRUE.
        CASE (64)           
          errnam = "NC_ETRUNC       "
          report = "File likely truncated or possibly corrupted"
          fatal = .TRUE.
        CASE (101)
          errnam = "NC_EHDFERR      "
          fatal = .TRUE.
        CASE (102)
          errnam = "NC_ECANTREAD    "
          fatal = .TRUE.
        CASE (103)
          errnam = "NC_ECANTWRITE   "
          fatal = .TRUE.
        CASE (104)
          errnam = "NC_ECANTCREATE  "
          fatal = .TRUE.
        CASE (105)
          errnam = "NC_EFILEMETA    "
          fatal = .TRUE.
        CASE (106)
          errnam = "NC_EDIMMETA     "
          fatal = .TRUE.
        CASE (107)
          errnam = "NC_EATTMETA     "
          fatal = .TRUE.
        CASE (108)
          errnam = "NC_EVARMETA     "
          fatal = .TRUE.
        CASE (109)
          errnam = "NC_ENOCOMPOUND  "
          fatal = .TRUE.
        CASE (110)
          errnam = "NC_EATTEXISTS   "
          fatal = .TRUE.
        CASE (111) 
          errnam = "NC_ENOTNC4      "
          report = "Attempting netcdf-4 operation on netcdf-3 file."
          fatal = .TRUE.
        CASE (112) 
          errnam = "NC_ESTRICTNC3   "
          report = 
     $    "Attempting netcdf-4 operation on strict nc3 netcdf-4 file."
          fatal = .TRUE.
        CASE (113) 
          errnam = "NC_EBADGRPID    "
          report = "Bad group id. Bad!"
          fatal = .TRUE.
        CASE (114) 
          errnam = "NC_EBADTYPEID   "
          report = "Bad type id."
          fatal = .TRUE.
        CASE (115) 
          errnam = "NC_EBADFIELDID  "
          report = "Bad field id."
          fatal = .TRUE.
        CASE (116)
          errnam = "NC_EUNKNAME     "
          fatal = .TRUE.
        CASE DEFAULT
          status = -(status)
          errnam = "NC_UNDEFINED_ERR"
          report = "Undefined error"
          fatal = .TRUE.
      END SELECT

      WRITE(lout,*) "NetCDF error: ",text,status,errnam," ",report
      PRINT*, "NetCDF error: ",text,status
      PRINT*, errnam," : ",report
! for non-fatal errors, pause execution
      IF (.NOT.fatal) THEN 
          PRINT *, "Hit <Enter> to continue"
          READ(5,*) 
! for fatal errors, stop execution
      ELSE
        STOP
      ENDIF

      END SUBROUTINE eznc_handle_err

* -----------------------------------------------------
      SUBROUTINE open_ncfile_new(filename,ncid)

! create a new netcdf file (replace any existing file)
! file is automatically created in DEFINE mode
!------------------------------------------------------
! cmode : OPTIONS for dealing with an existing file of same name:
! NF90_CLOBBER = overwrite (replace) existing file
! NF90_NOCLOBBER = return an error and stop IF file already exists
!-----------------
! other options :
! NF90_SHARE = allow different processes to read / write simulataneously
! NF90_64BIT_OFFSET = create 64-bit offest format file (useful for >2GB)
! NF90_HDF5 = output is in netCDF-4/HDF5 format
!-----------------

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      CHARACTER(*) filename
! OUTPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

!      print*,"open_ncfile_new : ",filename,ncid

      text   ="NF90_CREATE, "//filename
      status = NF90_CREATE(path=filename,
     &                     cmode=NF90_CLOBBER,
     &                     ncid=ncid)
!      status = NF90_CREATE(path=filename,
!     &                     cmode=OR(NF90_CLOBBER,NF90_64BIT_OFFSET),
!     &                     ncid=ncid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! file automatically CREATEd in DEFINE mode: no need to switch

      END SUBROUTINE open_ncfile_new

* -------------------------------
      SUBROUTINE open_ncfile_readonly(filename,ncid)

! open an existing netcdf file for reading only (in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      CHARACTER(*) filename
! OUTPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

      text   ="NF90_OPEN, "//filename
      status = NF90_OPEN(path=filename,mode=NF90_NOWRITE,ncid=ncid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

!      print*,"open_ncfile_readonly : ",filename,ncid

! switch to DATA mode just in case it's not already there
      CALL switch_ncfile_to_data_mode(ncid)

      END SUBROUTINE open_ncfile_readonly

* -------------------------------
      SUBROUTINE open_ncfile_readwrite(filename,ncid)

! open an existing netcdf file for reading and writing (in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      CHARACTER(*) filename
! OUTPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

!      print*,"open_ncfile_readwrite : ",filename,ncid

      text   ="NF90_OPEN, "//filename
      status = NF90_OPEN(path=filename,mode=NF90_WRITE,ncid=ncid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! switch to DEFINE mode just in case it's not already there
      CALL switch_ncfile_to_define_mode(ncid)

      END SUBROUTINE open_ncfile_readwrite

* -------------------------------
      SUBROUTINE close_ncfile(ncid)

! close an open netcdf file 

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

!      print*,"close_ncfile : ",ncid

      text   ="NF90_CLOSE"
      status = NF90_CLOSE(ncid=ncid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE close_ncfile

* -------------------------------
      SUBROUTINE switch_ncfile_to_data_mode(ncid)

! switch an open netcdf file to DATA mode, 
! allowing values to be written

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

!      print*,"switch_ncfile_to_data_mode : ",ncid

      text   ="NF90_ENDDEF"
      status = NF90_ENDDEF(ncid=ncid)
      IF (status.EQ.-38) RETURN  ! already in DATA mode, no action needed
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE switch_ncfile_to_data_mode

* -------------------------------
      SUBROUTINE switch_ncfile_to_define_mode(ncid)

! switch an open netcdf file to DEFINE mode, 
! allowing array dimensions or new variables to be specifed

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(25) text

!      print*,"switch_ncfile_to_define_mode : ",ncid

      text   ="NF90_REDEF"
      status = NF90_REDEF(ncid=ncid)
      IF (status.EQ.-39) RETURN  ! already in DEFINE mode, no action needed
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE switch_ncfile_to_define_mode

* -------------------------------
      SUBROUTINE eznc_def_globalatt(ncid,AttName,AttTxt)

! define global attribute using specified text ("AttTxt")
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) AttName
      CHARACTER(*) AttTxt
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(40) text

      text   ="NF90_PUT_ATT "//AttName//" = "//
     &         AttTxt
      PRINT*,text
      status = NF90_PUT_ATT(ncid,NF90_GLOBAL,AttName,AttTxt)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_PUT_ATT(ncid,NF90_GLOBAL,AttName,AttTxt)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_globalatt

* -------------------------------
      SUBROUTINE eznc_def_sysglobatt(ncid,AttName,Command)

! define global attribute using results of a system call
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) Command
      !CHARACTER(*) SysCall
      CHARACTER(*) AttName
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(60) AttTxt
      CHARACTER(60) text
      CHARACTER(60) SysCall

      SysCall = 'echo '//Command//' > dummy.99'
      CALL SYSTEM(SysCall)

      OPEN(99,file='dummy.99')
        READ(99,'(a60)') AttTxt
        PRINT*,"AttTxt = ",AttTxt
      CLOSE(99)

      text   ="NF90_PUT_ATT  "//AttName
      status = NF90_PUT_ATT(ncid,NF90_GLOBAL,AttName,AttTxt)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_PUT_ATT(ncid,NF90_GLOBAL,AttName,AttTxt)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_sysglobatt

* -------------------------------
      SUBROUTINE eznc_copy_globalatt(ncid_in,AttName,ncid)

! copy global attribute from one dataset to another
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid_in,ncid
      CHARACTER(*) AttName
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(40) text

! need a checker to make sure variable exists...

      text   ="NF90_COPY_ATT "//AttName
      status = NF90_COPY_ATT(ncid_in,NF90_GLOBAL,AttName,
     &                       ncid,NF90_GLOBAL)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_COPY_ATT(ncid_in,NF90_GLOBAL,AttName,
     &                         ncid,NF90_GLOBAL)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_copy_globalatt

* -------------------------------
      SUBROUTINE eznc_rename_globalatt(ncid,OldName,NewName)

! rename existing global attribute 
! note that dataset must be in DEFINE mode to do this, unless
! len(Name_old) /< len(Name_new)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) OldName
      CHARACTER(*) NewName
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(40) text

      text   ="NF90_RENAME_ATT "//NewName
      status = NF90_RENAME_ATT(ncid,NF90_GLOBAL,OldName,NewName)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_RENAME_ATT(ncid,NF90_GLOBAL,OldName,NewName)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_rename_globalatt

* -------------------------------
      SUBROUTINE eznc_del_globalatt(ncid,AttName)

! delete an existing global attribute to allow writing of a new one
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) AttName
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(40) text

      text   ="NF90_DEL_ATT "//AttName
      status = NF90_DEL_ATT(ncid,NF90_GLOBAL,AttName)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_DEL_ATT(ncid,NF90_GLOBAL,AttName)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_del_globalatt

* -------------------------------
      SUBROUTINE eznc_def_localatt(ncid,VarName,AttName,AttTxt)

! define variable-specific attribute using specified text ("AttTxt")
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
      CHARACTER(*) AttName
      CHARACTER(*) AttTxt
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(40) text

! find variable ID from its name
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! assuming variable exists, apply attribute to it
      text   ="NF90_PUT_ATT "//AttName
      status = NF90_PUT_ATT(ncid,varid,AttName,AttTxt)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_PUT_ATT(ncid,varid,AttName,AttTxt)
      ENDIF

      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_localatt

* -------------------------------
      SUBROUTINE eznc_def_dim(ncid,DimName,dim_in)

! define a dimension, given its name and value
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER,INTENT(IN) :: dim_in
      CHARACTER(*) DimName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER dimid
      INTEGER dimval
      CHARACTER(25) text

      IF(dim_in.EQ.0) THEN
         dimval = 1
      ELSE
         dimval = dim_in
      ENDIF
!      print*,"eznc_def_dim:",DimName,DimVal

! write dimension
      text   ="NF90_DEF_DIM "//DimName
      status = NF90_DEF_DIM(ncid,DimName,dimval,dimid)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_DEF_DIM(ncid,DimName,dimval,dimid)
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_dim

* -------------------------------
      SUBROUTINE eznc_def_dim_unlim(ncid,DimName)

! define an UNLIMITED dimension, given its name
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) DimName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER dimid
      CHARACTER(25) text

!      print*,"eznc_def_dim_unlim:",DimName

! write dimension
      text   ="NF90_DEF_DIM "//DimName
      status = NF90_DEF_DIM(ncid,DimName,NF90_UNLIMITED,dimid)

! if file is in DATA mode, switch to DEFINE mode and try again
      IF (status.EQ.-38) THEN  
        CALL switch_ncfile_to_define_mode(ncid)
        status = NF90_DEF_DIM(ncid,DimName,NF90_UNLIMITED,dimid)
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_dim_unlim

* -------------------------------
      SUBROUTINE eznc_def_0Dint(ncid,VarName)

! define a scalar integer, given its name
! dataset must be in DEFINE mode
! write the value later, using subroutine ncput_0Dint 

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(25) text

!      print*,"eznc_def_0Dint:",VarName

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_INT,varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_INT,varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_0Dint

* -------------------------------
      SUBROUTINE eznc_def_0Dreal(ncid,VarName)

! define a scalar real, given its name
! dataset must be in DEFINE mode
! write the value later, using subroutine ncput_0Dreal

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(25) text

!      print*,"eznc_def_0Dreal:",VarName

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_FLOAT,varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_FLOAT,varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_0Dreal

* -------------------------------
      SUBROUTINE eznc_def_1Dint(ncid,VarName,DimName)

! define a 1-D integer array, given its name and dimension name 
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_1Dint

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
      CHARACTER(*) DimName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid
      CHARACTER(25) text

!      print*,"eznc_def_1Dint:",VarName,"(",DimName,")"

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimid of named dimension
        text   ="NF90_INQ_DIMID "//DimName
        status = NF90_INQ_DIMID(ncid,DimName,dimid)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_INT,(/dimid/),varid)

! > if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_INT,(/dimid/),varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_1Dint

* -------------------------------
      SUBROUTINE eznc_def_1Dreal(ncid,VarName,DimName)

! define a 1-D array of reals
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_1Dreal

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
      CHARACTER(*) DimName
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid
      CHARACTER(25) text

!      print*,"eznc_def_1Dreal:",VarName,"(",DimName,")"

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimid of named dimension
        text   ="NF90_INQ_DIMID "//DimName
        status = NF90_INQ_DIMID(ncid,DimName,dimid)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_FLOAT,(/dimid/),varid)

! > if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_FLOAT,(/dimid/),varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_1Dreal

* -------------------------------
      SUBROUTINE eznc_def_1Dchar(ncid,VarName,DimName1,DimName2)

! define a 1-D array (size Dim2) of character strings with length Dim1
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2
      CHARACTER(25) text

!      print*,"eznc_def_1Dchar:",VarName

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_CHAR,
     $                       (/dimid1,dimid2/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_CHAR,
     $                         (/dimid1,dimid2/),varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_1Dchar

* -------------------------------
      SUBROUTINE eznc_def_2Dint(ncid,VarName,DimName1,DimName2)

! define a 2-D array of integers, given name and dimension names
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_2Dint

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2
      CHARACTER(25) text


!      print*,"eznc_def_2Dint:",VarName,"(",DimName1,",",DimName2,")"

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_INT,
     &                       (/dimid1,dimid2/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_INT,
     &                         (/dimid1,dimid2/),varid)
        ENDIF
      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_2Dint

* -------------------------------
      SUBROUTINE eznc_def_2Dreal(ncid,VarName,DimName1,DimName2)

! define a 2-D array of reals, given name and dimension names
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_2Dreal

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2
      CHARACTER(25) text

!      print*,"eznc_def_2Dreal:",VarName,"(",DimName1,",",DimName2,")"

! check if variable already defined. 
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_REAL,
     &                       (/dimid1,dimid2/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_REAL,
     &                         (/dimid1,dimid2/),varid)
        ENDIF
      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_2Dreal

* -------------------------------
      SUBROUTINE eznc_def_2Ddbl(ncid,VarName,DimName1,DimName2)

! USE SPARINGLY !
! define a 2-D array of DP values, given name and dimension names
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_2Ddbl

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2
      CHARACTER(25) text

!      print*,"eznc_def_2Ddbl:",VarName

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_DOUBLE,
     &                       (/dimid1,dimid2/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_DOUBLE,
     &                         (/dimid1,dimid2/),varid)
        ENDIF
      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_2Ddbl

* -------------------------------
      SUBROUTINE eznc_def_2Dchar(ncid,VarName,
     &                           DimName1,DimName2,DimName3)

! define a 2-D array (size Dim2,Dim3) of character strings with length Dim1
! (dataset must be in DEFINE mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2,DimName3
! OUTPUT VARIABLES
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2,dimid3
      CHARACTER(25) text

!      print*,"eznc_def_2Dchar:",VarName

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName3
        status = NF90_INQ_DIMID(ncid,DimName3,dimid3)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_CHAR,
     $                       (/dimid1,dimid2,dimid3/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_CHAR,
     $                         (/dimid1,dimid2,dimid3/),varid)
        ENDIF

      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_2Dchar

* -------------------------------
      SUBROUTINE eznc_def_3Dint(ncid,VarName,
     &                          DimName1,DimName2,DimName3)

! define a 3-D array of integers, given name and dimension names
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_2Dint

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2,DimName3
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2,dimid3
      CHARACTER(25) text


!      print*,"eznc_def_3Dint:",VarName
!     &      ,"(",DimName1,",",DimName2,",",DimName3,")"

! check if variable already defined. If yes, return
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName3
        status = NF90_INQ_DIMID(ncid,DimName3,dimid3)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_INT,
     &                       (/dimid1,dimid2,dimid3/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_INT,
     &                         (/dimid1,dimid2,dimid3/),varid)
        ENDIF
      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_3Dint

* -------------------------------
      SUBROUTINE eznc_def_3Dreal(ncid,VarName,
     &                           DimName1,DimName2,DimName3)

! define a 3-D array of reals, given name and dimension names
! dataset must be in DEFINE mode
! write the values later, using subroutine eznc_put_2Dreal

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName,DimName1,DimName2,DimName3
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER dimid1,dimid2,dimid3
      CHARACTER(25) text

!!      print*,"eznc_def_3Dreal:",VarName,"(",DimName1,",",DimName2,",",
!     &                                      DimName3,")"

! check if variable already defined. 
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)

! if variable not found, define it
      IF (status.EQ.-49) THEN 

! > find dimids of named dimensions
        text   ="NF90_INQ_DIMID "//DimName1
        status = NF90_INQ_DIMID(ncid,DimName1,dimid1)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName2
        status = NF90_INQ_DIMID(ncid,DimName2,dimid2)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

        text   ="NF90_INQ_DIMID "//DimName3
        status = NF90_INQ_DIMID(ncid,DimName3,dimid3)
        IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! > define variable
        text   ="NF90_DEF_VAR "//VarName
        status = NF90_DEF_VAR(ncid,VarName,NF90_REAL,
     &                       (/dimid1,dimid2,dimid3/),varid)

! if file is in DATA mode, switch to DEFINE mode and try again
        IF (status.EQ.-38) THEN  
          CALL switch_ncfile_to_define_mode(ncid)
          status = NF90_DEF_VAR(ncid,VarName,NF90_REAL,
     &                         (/dimid1,dimid2,dimid3/),varid)
        ENDIF
      ENDIF

! Handle any other inquiry errors
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_def_3Dreal

* -------------------------------
      SUBROUTINE eznc_put_0Dint(ncid,VarName,value)

! write a value to a previously-defined scalar integer
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER value
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(25) text

!      print*,"eznc_put_0Dint:",VarName,value

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,value)

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,value)
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dint

* -------------------------------
      SUBROUTINE eznc_put_0Dreal(ncid,VarName,value)

! write a value to a previously-defined scalar real number
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      REAL    value
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(25) text

!      print*,"eznc_put_0Dreal:",VarName,value

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,value)

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,value)
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dreal

* -------------------------------
      SUBROUTINE eznc_put_1Dint(ncid,VarName,invals,start1,end1)

! write values to a previously-defined 1-D integer array 
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1,invals(end1-start1+1)
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(25) text

!      print*,"eznc_put_1Dint:",VarName

      count1=end1-start1+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1/),
     $                      count=(/count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals(start1:end1),
     $                        start=(/start1/),
     $                        count=(/count1/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dint

* -------------------------------
      SUBROUTINE eznc_put_1Dreal(ncid,VarName,invals,start1,end1)

! write values to a previously-defined 1-D real array 
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1
      REAL,DIMENSION(end1-start1+1) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(25) text

!      print*,"eznc_put_1Dreal:",VarName !,invals

      count1=end1-start1+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals(start1:end1),
     $                      start=(/start1/),
     $                      count=(/count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals(start1:end1),
     $                        start=(/start1/),
     $                        count=(/count1/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dreal

* -------------------------------
      SUBROUTINE eznc_put_1Dchar(ncid,VarName,values,nchar,start1,end1)

! write values to a previously-defined 1-D array of string variables
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER nchar,start1,end1
      CHARACTER(nchar) values(end1-start1+1)
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(25) text

!      print*,"eznc_put_1Dchar:",VarName
      count1=end1-start1+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,values,
     $                      start=(/1,start1/),
     $                      count=(/nchar,count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,values,
     $                        start=(/1,start1/),
     $                        count=(/nchar,count1/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dchar

* -------------------------------
      SUBROUTINE eznc_put_2Dint(ncid,VarName,invals,
     $                       start1,end1,start2,end2)

! write values to a previously-defined 2-D integer array
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2,end1,end2
      INTEGER,DIMENSION((end1-start1+1),(start2-end2+1)) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(25) text

!      print*,"eznc_put_2Dint:",VarName

      count1=end1-start1+1
      count2=end2-start2+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals,
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_2Dint

* -------------------------------
      SUBROUTINE eznc_put_2Dreal(ncid,VarName,invals,
     $                       start1,end1,start2,end2)

! write values to a previously-defined 2-D real array
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2,end1,end2
      REAL,DIMENSION((end1-start1+1),(start2-end2+1)) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(25) text

!      print*,"eznc_put_2Dreal:",VarName

      count1=end1-start1+1
      count2=end2-start2+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals,
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_2Dreal

* -------------------------------
      SUBROUTINE eznc_put_2Dchar(ncid,VarName,values,nchar,
     &                           start1,end1,start2,end2)

! write values to a previously-defined 1-D array of string variables
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER nchar,start1,end1,start2,end2
      CHARACTER(nchar) values(end1-start1+1,end2-start2+1)
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(25) text

!      print*,"eznc_put_2Dchar:",VarName
      count1=end1-start1+1
      count2=end2-start2+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,values,
     $                        start=(/1,start1,start2/),
     $                      count=(/nchar,count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,values,
     $                        start=(/1,start1,start2/),
     $                      count=(/nchar,count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_2Dchar

* -------------------------------
      SUBROUTINE eznc_put_3Dint(ncid,VarName,invals,
     $                       start1,end1,start2,end2,start3,end3)

! write values to a previously-defined 3-D integer array
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2,start3,end1,end2,end3
      INTEGER,DIMENSION((end1-start1+1),(start2-end2+1),
     &                  (start3-end3+1)) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2,count3
      CHARACTER(25) text

!      print*,"eznc_put_3Dint:",VarName

      count1=end1-start1+1
      count2=end2-start2+1
      count3=end3-start3+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1,start2,start3/),
     $                      count=(/count1,count2,count3/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals,
     $                        start=(/start1,start2,start3/),
     $                        count=(/count1,count2,count3/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_3Dint

* -------------------------------
      SUBROUTINE eznc_put_3Dreal(ncid,VarName,invals,
     $                       start1,end1,start2,end2,start3,end3)

! write values to a previously-defined 3-D real array
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2,start3,end1,end2,end3
      REAL,DIMENSION((end1-start1+1),(start2-end2+1),(start3-end3+1)) 
     &                        :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2,count3
      CHARACTER(25) text

!      print*,"eznc_put_3Dreal:",VarName

      count1=end1-start1+1
      count2=end2-start2+1
      count3=end3-start3+1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1,start2,start3/),
     $                      count=(/count1,count2,count3/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,invals,
     $                      start=(/start1,start2,start3/),
     $                      count=(/count1,count2,count3/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_3Dreal

* -------------------------------
* -------------------------------
!      SUBROUTINE eznc_put_2Ddbl(........
!
!! CAUTION !! DO NOT USE - DEFINE AS NF90_DOUBLE BUT PUT AS "REAL"
!             UNDER COMPILATION OPTION real-8
!
* -------------------------------
      SUBROUTINE eznc_put_0Dint_into1D(ncid,VarName,inval,start1)

! write a scalar value to a previously-defined 1-D real array 
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1
      INTEGER :: inval
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
!      REAL,DIMENSION(start1) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_0Dint_into1D:",VarName,inval

! convert scalar into 1-D array entry
!      ncvals(start1)=inval
      count1=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                      start=(/start1/),
     $                      count=(/count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                        start=(/start1/),
     $                        count=(/count1/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dint_into1D

* -------------------------------
      SUBROUTINE eznc_put_0Dreal_into1D(ncid,VarName,inval,start1)

! write a scalar value to a previously-defined 1-D real array 
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1
      REAL :: inval
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
!      REAL,DIMENSION(start1) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_0Dreal_into1D:",VarName,inval

! convert scalar into 1-D array entry
!      ncvals(start1)=inval
      count1=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                      start=(/start1/),
     $                      count=(/count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                        start=(/start1/),
     $                        count=(/count1/))
      ENDIF
! if number not representable, record error and return
      IF (status.EQ.-60) RETURN

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dreal_into1D

* -------------------------------
      SUBROUTINE eznc_put_0Dchar_into1D(ncid,VarName,inval,nchar,start1)

! write CHARACTER data item into a previously-defined 1-D array of
! chars as ncvals(1:nchar,i)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER nchar,start1
      CHARACTER(LEN=nchar) :: inval
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(LEN=nchar),DIMENSION(start1) :: ncval
      CHARACTER(25) text

!      print*,"eznc_put_0Dchar_into1D:",VarName,"   ",inval
      !print*,nchar,start1

! convert 0-D array into 1-D array slab of dimension (1)
      ncval(start1) = inval

      count1=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,
     $                      ncval(start1:start1),
     $                      start=(/1,start1/),
     $                      count=(/nchar,count1/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,
     $                        ncval(start1:start1),
     $                        start=(/1,start1/),
     $                        count=(/nchar,count1/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dchar_into1D

* -------------------------------
      SUBROUTINE eznc_put_0Dreal_into2D(ncid,VarName,inval,
     &                                       start1,start2)

! write a scalar value to a previously-defined 2-D real array 
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2
      REAL :: inval
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(25) text

!      print*,"eznc_put_0Dreal_into2D:",VarName,inval

! convert scalar into 2-D array entry
      count1=1
      count2=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,(/inval/),
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_0Dreal_into2D

* -------------------------------
      SUBROUTINE eznc_put_1Dint_into2D(ncid,VarName,invals,
     $                                 start1,end1,start2)

! write 1D INTEGER data slab into a previously-defined 2-D integer array
! as ncvals(islab,start:end)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1,start2
      INTEGER,DIMENSION(end1-start1+1) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      INTEGER,DIMENSION(end1-start1+1,start2) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_1Dreal_into2D:",VarName
      !print*,invals
      !print*,start1,end1,start2

! convert 1-D array into 2-D array slab of dimensions (count1,1)
      ncvals(start1:end1,start2) = invals

      count1=end1-start1+1
      count2=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,
     $                      ncvals(start1:end1,start2:start2),
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,
     $                        ncvals(start1:end1,start2:start2),
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dint_into2D

* -------------------------------
      SUBROUTINE eznc_put_1Dreal_into2D(ncid,VarName,invals,
     $                               start1,end1,start2)

! write 1D REAL data slab into a previously-defined 2-D real array
! as ncvals(islab,start:end)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1,start2
      REAL,DIMENSION(end1-start1+1) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      REAL,DIMENSION(end1-start1+1,start2) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_1Dreal_into2D:",VarName
      !print*,invals
      !print*,start1,end1,start2

! convert 1-D array into 2-D array slab of dimensions (count1,1)
      ncvals(start1:end1,start2) = invals

      count1=end1-start1+1
      count2=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,
     $                      ncvals(start1:end1,start2:start2),
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,
     $                        ncvals(start1:end1,start2:start2),
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dreal_into2D

* -------------------------------
      SUBROUTINE eznc_put_1Dreal_into2D_mask(ncid,VarName,invals,
     $                               start1,end1,start2,maskval)

! write 1D REAL data slab into a previously-defined 2-D real array
! as ncvals(islab,start:end)
! (dataset must be in DATA mode)
! Writes ZERO ("0.") for values under threshold "maskval"

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1,start2
      REAL,DIMENSION(end1-start1+1) :: invals
      REAL    maskval
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      REAL,DIMENSION(end1-start1+1,start2) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_1Dreal_into2D_mask:",VarName,varid,maskval
      !print*,invals

! zero out undesirable values from 1-D input array
      WHERE (invals.LT.maskval) invals=0.

! convert 1-D array into 2-D array slab of dimensions (count1,1)
      ncvals(start1:end1,start2) = invals

      count1=end1-start1+1
      count2=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid,
     $                      ncvals(start1:end1,start2:start2),
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,
     $                        ncvals(start1:end1,start2:start2),
     $                        start=(/start1,start2/),
     $                        count=(/count1,count2/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dreal_into2D_mask

* -------------------------------
      SUBROUTINE eznc_put_1Dreal_into3D(ncid,VarName,invals,
     $                               start1,end1,start2,start3)

! write 1D REAL data slab into a previously-defined s32-D real array
! as ncvals(islab,start:end)
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,end1,start2,start3
      REAL,DIMENSION(end1-start1+1) :: invals
      CHARACTER(*) VarName
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2,count3
      REAL,DIMENSION(end1-start1+1,start2,start3) :: ncvals
      CHARACTER(25) text

!      print*,"eznc_put_1Dreal_into3D:",VarName
      !print*,invals

! convert 1-D array into 3-D array slab of dimensions (count1,1,1)
      ncvals(start1:end1,start2,start3) = invals

      count1=end1-start1+1
      count2=1
      count3=1

! find variable ID
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! write variable
      text   ="NF90_PUT_VAR "//VarName
      status = NF90_PUT_VAR(ncid,varid, 
     $                  ncvals(start1:end1,start2:start2,start3:start3),
     $                  start=(/start1,start2,start3/),
     $                  count=(/count1,count2,count3/))

! if file is in DEFINE mode, switch to DATA mode and try again
      IF (status.EQ.-39) THEN  
        CALL switch_ncfile_to_data_mode(ncid)
        status = NF90_PUT_VAR(ncid,varid,
     $                  ncvals(start1:end1,start2:start2,start3:start3),
     $                  start=(/start1,start2,start3/),
     $                  count=(/count1,count2,count3/))
      ENDIF

      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_put_1Dreal_into3D
* -------------------------------
      SUBROUTINE eznc_get_globalatt(ncid,AttName,AttTxt)

! retrieve text of global attribute using attribute name

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) AttName
      CHARACTER(*) AttTxt
! INTERNAL VARIABLES
      INTEGER status
      CHARACTER(40) text

      !print*,"eznc_get_globalatt : ",AttName,ncid

! find global attribute
      text   ="NF90_GET_ATT "//Attname
      status = NF90_GET_ATT(ncid,NF90_GLOBAL,AttName,AttTxt)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_get_globalatt

* -------------------------------
      SUBROUTINE eznc_get_localatt(ncid,VarName,AttName,AttTxt)

! retrieve attribute text using variable and attribute names

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
      CHARACTER(*) AttName
      CHARACTER(*) AttTxt
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(40) text

!      print*,"eznc_get_localatt : ",VarName," ",AttName,ncid

! find variable ID from its name
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

! assuming variable exists, find its attribute
      text   ="NF90_GET_ATT "//AttName
      status = NF90_GET_ATT(ncid,varid,AttName,AttTxt)
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      END SUBROUTINE eznc_get_localatt

* -------------------------------
      SUBROUTINE eznc_get_dimension(ncid,DimName,dimval)

! read the value of a dimension whose name is already known
! (dataset can be in either DEFINE or DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER :: ncid
      CHARACTER(*) :: DimName
! OUTPUT VARIABLES
      INTEGER :: dimval
! INTERNAL VARIABLES
      INTEGER :: status
      INTEGER :: dimid
      CHARACTER(40) :: text

      text   ="NF90_INQ_DIMID "//DimName
      status = NF90_INQ_DIMID(ncid,DimName,dimid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      text   ="NF90_INQUIRE_DIMENSION "//DimName
      status = NF90_INQUIRE_DIMENSION(ncid,dimid,len=dimval)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

!      print*,"eznc_get_dimension: ",DimName,dimval

      END SUBROUTINE eznc_get_dimension

* -------------------------------
      SUBROUTINE eznc_get_dimid(ncid,DimName,dimid)

! read the I.D (only) of a dimension whose name is already known
! (dataset can be in either DEFINE or DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER :: ncid
      CHARACTER(*) :: DimName
! OUTPUT VARIABLES
      INTEGER :: dimid
! INTERNAL VARIABLES
      INTEGER :: status
      CHARACTER(40) :: text


      text   ="NF90_INQ_DIMID "//DimName
      status = NF90_INQ_DIMID(ncid,DimName,dimid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

!      print*,"eznc_get_dimid: ", DimName,": dimid=",dimid

      END SUBROUTINE eznc_get_dimid

* -------------------------------
      SUBROUTINE eznc_get_0Dint(ncid,VarName,value)

! read the value of a scalar integer whose name is already known
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      INTEGER value
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(40) text

!      print*,"eznc_get_0Dint: ",VarName

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,value)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      !print*,"VarName=",VarName,": varid=",varid,": Value=",value
      !print*,"eznc_get_0Dint: ",VarName,value

      END SUBROUTINE eznc_get_0Dint

* -------------------------------
      SUBROUTINE eznc_get_0Dreal(ncid,VarName,value)

! read the value of a scalar integer whose name is already known
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      REAL value
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      CHARACTER(40) text

!      print*,"eznc_get_0Dreal: ",VarName

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,value)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      !print*,"VarName=",VarName,": varid=",varid,": Value=",value
      !print*,"eznc_get_0Dreal: ",VarName,value

      END SUBROUTINE eznc_get_0Dreal

* -------------------------------
      SUBROUTINE eznc_get_1Dint(ncid,VarName,outvals,start1,end1)
! read a 1-D integer array whose name & dimension are already known
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid,start1,end1
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      INTEGER,DIMENSION(end1-start1+1) :: outvals
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(40) text

!      print*,"eznc_get_1Dint: ",VarName

      outvals(:) = 0
      count1=end1-start1+1

      IF(count1.LT.1) RETURN

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,outvals,
     $                      start=(/start1/),
     $                      count=(/count1/))
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      !print*,"VarName=",VarName,": varid=",varid,": values=",outvals
      !print*,outvals

      END SUBROUTINE eznc_get_1Dint

* -------------------------------
      SUBROUTINE eznc_get_1Dreal(ncid,VarName,outvals,start1,end1)

! read a 1-D real array whose name & dimension are already known
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid,start1,end1
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      REAL,DIMENSION(end1-start1+1) :: outvals
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      CHARACTER(40) text

!      print*,"eznc_get_1Dreal: ",VarName

      outvals(:) = 0.
      count1=end1-start1+1

      IF(count1.LT.1) RETURN

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,outvals,
     $                      start=(/start1/),
     $                      count=(/count1/))
      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)

      !print*,outvals

      END SUBROUTINE eznc_get_1Dreal

* -------------------------------
      SUBROUTINE eznc_get_1Dchar(ncid,VarName,outvals,
     $                                nchar,start1,end1)

! read a 1-D array of character strings, 
! whose name & dimensions are already known
! NOTE THAT THIS SUBROUTINE EXPECTS TO START AT LIST ITEM (1)
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid,nchar,start1,end1
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      CHARACTER(nchar) outvals(end1-start1+1)
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1
      INTEGER,PARAMETER :: firstchar = 1
      CHARACTER(40) text

!      print*,"eznc_get_1Dchar: ",VarName

      count1=end1-start1+1

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,outvals,
     $                      start=(/firstchar,start1/),
     $                      count=(/nchar,count1/))
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)


      !print*,"VarName=",VarName,": varid=",varid,": values=",values
      !print*,values

      END SUBROUTINE eznc_get_1Dchar

* -------------------------------
      SUBROUTINE eznc_get_2Dint(ncid,VarName,outvals,
     $                     start1,end1,start2,end2)

! read a 2-D integer array whose name & dimensions are already known
! dimension values are used in variable declaration
! THIS FORM USED FOR RETRIEVING SUBSECTION OF STORED ARRAY
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2
      INTEGER end1,end2
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      INTEGER,DIMENSION((end1-start1+1),(end2-start2+1)) :: outvals
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(40) text
      INTEGER, PARAMETER ::  stride1=1, stride2=1

!      print*,"eznc_get_2Dint: ",VarName

      outvals(:,:) = 0
      count1=end1-start1+1
      count2=end2-start2+1

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      STATUS = NF90_INQ_VARID(ncid,VarName,varid)
      IF (STATUS/=NF90_NOERR) call eznc_handle_err(STATUS,text)

!      PRINT*, 'eznc_get_2Dint: ', varname ,count1,count2

      IF(count1.LT.1.OR.count2.LT.1) RETURN

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,
     $                      outvals,
     $                      start=(/start1,start2/),
     $                      count=(/count1,count2/),
     $                      stride=(/stride1,stride2/))
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      END SUBROUTINE eznc_get_2Dint

* -------------------------------
      SUBROUTINE eznc_get_2Dreal(ncid,VarName,outvals,
     $                               start1,end1,start2,end2)

! read a 2-D real array whose name & dimensions are already known
! THIS FORM USED FOR RETRIEVING SUBSECTION OF STORED ARRAY
! "start" and "end" refer to the indices of the STORED array, 
! not the subsection.
!!!! Remember that the dimensions are quoted IN REVERSE ORDER 
!    in the NetCDF file !!!
! (dataset must be in DATA mode)

      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2
      INTEGER end1,end2
      CHARACTER(*) VarName
! OUTPUT VARIABLES
      REAL,DIMENSION((end1-start1+1),(end2-start2+1)) :: outvals
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1,count2
      CHARACTER(40) text
      INTEGER, PARAMETER :: stride1=1, stride2=1

      outvals(:,:) = 0
      count1=end1-start1+1
      count2=end2-start2+1

!      print*,"eznc_get_2Dreal:",VarName,count1,count2

      IF(count1.LT.1.OR.count2.LT.1) RETURN

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,
     $                    outvals,
     $                    start=(/start1,start2/),
     $                    count=(/count1,count2/))
!     $                    stride=(/stride1,stride2/))
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)


      !print*,outvals

      END SUBROUTINE eznc_get_2Dreal

* ----------------------------
      SUBROUTINE eznc_get_3Dreal(ncid, VarName, outvals,
     &                  start1, end1, start2, end2, start3, end3)
! read a 3-D real array whose name & dimensions are already known
! THIS FORM USED FOR RETRIEVING SUBSECTION OF STORED ARRAY
! "start" and "end" refer to the indices of the STORED array, 
! not the subsection.
!!!! Remember that the dimensions are quoted IN REVERSE ORDER 
!    in the NetCDF file !!!
! (dataset must be in DATA mode)
      USE netcdf
      IMPLICIT NONE

! INPUT VARIABLES
      INTEGER ncid
      INTEGER start1,start2, start3
      INTEGER end1,end2, end3
      CHARACTER(*) VarName

! OUTPUT VARIABLES
      REAL,DIMENSION(end1-start1+1,end2-start2+1,end3-start3+1) ::
     &               outvals
! INTERNAL VARIABLES
      INTEGER status
      INTEGER varid
      INTEGER count1, count2, count3
      CHARACTER(40) text
      
!      print*,"eznc_get_3Dreal: ",VarName
      count1=end1-start1+1
      count2=end2-start2+1
      count3=end3-start3+1
!      print*,start1,start2,start3
!      print*,count1,count2,count3
      
      IF(count1.LT.1.OR.count2.LT.1.or.count3.LT.1) RETURN

      outvals(:,:,:) = 0

! find ID of variable
      text   ="NF90_INQ_VARID "//VarName
      status = NF90_INQ_VARID(ncid,VarName,varid)
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

! read variable
      text   ="NF90_GET_VAR "//VarName
      status = NF90_GET_VAR(ncid,varid,
     $                      outvals,
     $                      start=(/start1,start2,start3/),
     $                      count=(/count1,count2,count3/))
      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)

      
      END SUBROUTINE eznc_get_3Dreal

* -------------------------------
!-------------------------------------------------------!
!     BUFFER: USEFUL CODE SNIPPETS
!             USEFUL LINES FOR DEBUGGING
!-------------------------------------------------------!
* -------------------------------
!!-----find dimension ID------!
!      text   ="NF90_INQ_DIMID "//DimName
!      status = NF90_INQ_DIMID(ncid,DimName,dimid)
!      IF (status/=NF90_NOERR) CALL eznc_handle_err(status,text)
* -------------------------------
* -------------------------------
! TROUBLESHOOTING: 
!!-----investigate dimensions of known variable---------!
! VARIABLES FOR TROUBLESHOOTING
!      INTEGER :: numdims,dimsize,i
!      INTEGER,PARAMETER :: maxdims = 4, maxchars = 20
!      INTEGER,DIMENSION(maxdims) :: dimids
!      CHARACTER(maxchars) DimName
!
!      text   ="NF90_INQUIRE_VARIABLE "//VarName
!      status = NF90_INQUIRE_VARIABLE(ncid,varid,ndims=numDims,natts=i)
!      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)
!      text   ="NF90_INQUIRE_VARIABLE "//VarName
!      status = NF90_INQUIRE_VARIABLE(ncid,varid,dimids=dimids(:numDims))
!      IF (status/=NF90_NOERR) call eznc_handle_err(status,text)
!      DO i = 1,numdims
!        text   ="NF90_INQUIRE_DIMENSION "
!        status = NF90_INQUIRE_DIMENSION(ncid,dimids(i),
!     &                                  name=DimName,len=DimSize)
!        !print*,"dimension ",i,"=",dimname,dimsize
!      ENDDO

!-------------------------------------------------------!
